@@ -1,3 +1,4 @@
+
 /*
  * kernel/lvm_internal.h
  *
@@ -44,6 +45,7 @@ extern int loadtime;
 extern const char *const lvm_name;
 
 
+extern uint vg_count;
 extern vg_t *vg[];
 extern struct file_operations lvm_chr_fops;
 
@@ -53,6 +55,30 @@ extern struct file_operations lvm_blk_fops;
 extern struct block_device_operations lvm_blk_dops;
 #endif
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION ( 2, 4, 4)
+#define lvm_sectsize(dev) get_hardblocksize(dev)
+#else
+#define lvm_sectsize(dev) get_hardsect_size(dev)
+#endif
+
+/* 2.4.8 had no global min/max macros, and 2.4.9's were flawed */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 4, 10)
+
+#undef min
+#define min(x,y) ({ \
+	const typeof(x) _x = (x);       \
+	const typeof(y) _y = (y);       \
+	(void) (&_x == &_y);            \
+	_x < _y ? _x : _y; })
+
+#undef max
+#define max(x,y) ({ \
+	const typeof(x) _x = (x);       \
+	const typeof(y) _y = (y);       \
+	(void) (&_x == &_y);            \
+	_x > _y ? _x : _y; })
+
+#endif
 
 /* debug macros */
 #ifdef DEBUG_IOCTL
@@ -99,7 +125,7 @@ void lvm_fin_fs(void);
 
 void lvm_fs_create_vg(vg_t *vg_ptr);
 void lvm_fs_remove_vg(vg_t *vg_ptr);
-void lvm_fs_create_lv(vg_t *vg_ptr, lv_t *lv);
+devfs_handle_t lvm_fs_create_lv(vg_t *vg_ptr, lv_t *lv);
 void lvm_fs_remove_lv(vg_t *vg_ptr, lv_t *lv);
 void lvm_fs_create_pv(vg_t *vg_ptr, pv_t *pv);
 void lvm_fs_remove_pv(vg_t *vg_ptr, pv_t *pv);
