@@ -1,7 +1,7 @@
 /*
  * kernel/lvm.c
  *
- * Copyright (C) 1997 - 2001  Heinz Mauelshagen, Sistina Software
+ * Copyright (C) 1997 - 2002  Heinz Mauelshagen, Sistina Software
  *
  * February-November 1997
  * April-May,July-August,November 1998
@@ -1052,6 +1052,7 @@ static int lvm_blk_ioctl(struct inode *inode, struct file *file,
 		    (long) arg > LVM_MAX_READ_AHEAD)
 			return -EINVAL;
 		lv_ptr->lv_read_ahead = (long) arg;
+		LVM_CORRECT_READ_AHEAD(lv_ptr->lv_read_ahead);
 		read_ahead[MAJOR_NR] = lv_ptr->lv_read_ahead;
 		break;
 
@@ -1094,7 +1095,11 @@ static int lvm_blk_ioctl(struct inode *inode, struct file *file,
 	case LV_SET_ACCESS:
 		/* set access flags of a logical volume */
 		if (!capable(CAP_SYS_ADMIN)) return -EACCES;
+
+		down_write(&lv_ptr->lv_lock);
 		lv_ptr->lv_access = (ulong) arg;
+		up_write(&lv_ptr->lv_lock);
+
 		if ( lv_ptr->lv_access & LV_WRITE)
 			set_device_ro(lv_ptr->lv_dev, 0);
 		else
@@ -1107,7 +1112,9 @@ static int lvm_blk_ioctl(struct inode *inode, struct file *file,
 		if (!capable(CAP_SYS_ADMIN)) return -EACCES;
 		if (!((ulong) arg & LV_ACTIVE) && lv_ptr->lv_open > 1)
 			return -EPERM;
+		down_write(&lv_ptr->lv_lock);
 		lv_ptr->lv_status = (ulong) arg;
+		up_write(&lv_ptr->lv_lock);
 		break;
 
 	case LV_BMAP:
@@ -1122,7 +1129,9 @@ static int lvm_blk_ioctl(struct inode *inode, struct file *file,
 	case LV_SET_ALLOCATION:
 		/* set allocation flags of a logical volume */
 		if (!capable(CAP_SYS_ADMIN)) return -EACCES;
+		down_write(&lv_ptr->lv_lock);
 		lv_ptr->lv_allocation = (ulong) arg;
+		up_write(&lv_ptr->lv_lock);
 		break;
 
 	case LV_SNAPSHOT_USE_RATE:
@@ -1163,8 +1172,12 @@ static int lvm_get_snapshot_use_rate(lv_t *lv, void *arg)
 {
 	lv_snapshot_use_rate_req_t lv_rate_req;
 
-	if (!(lv->lv_access & LV_SNAPSHOT))
+	down_read(&lv->lv_lock);
+	if (!(lv->lv_access & LV_SNAPSHOT)) {
+		up_read(&lv->lv_lock);
 		return -EPERM;
+	}
+	up_read(&lv->lv_lock);
 
 	if (copy_from_user(&lv_rate_req, arg, sizeof(lv_rate_req)))
 		return -EFAULT;
@@ -1174,10 +1187,17 @@ static int lvm_get_snapshot_use_rate(lv_t *lv, void *arg)
 
 	switch (lv_rate_req.block) {
 	case 0:
+		down_write(&lv->lv_lock);
 		lv->lv_snapshot_use_rate = lv_rate_req.rate;
+		up_write(&lv->lv_lock);
+		down_read(&lv->lv_lock);
 		if (lv->lv_remap_ptr * 100 / lv->lv_remap_end <
-		    lv->lv_snapshot_use_rate)
+		    lv->lv_snapshot_use_rate) {
+			up_read(&lv->lv_lock);
 			interruptible_sleep_on(&lv->lv_snapshot_wait);
+			down_read(&lv->lv_lock);
+		}
+		up_read(&lv->lv_lock);
 		break;
 
 	case O_NONBLOCK:
@@ -1186,7 +1206,9 @@ static int lvm_get_snapshot_use_rate(lv_t *lv, void *arg)
 	default:
 		return -EINVAL;
 	}
+	down_read(&lv->lv_lock);
 	lv_rate_req.rate = lv->lv_remap_ptr * 100 / lv->lv_remap_end;
+	up_read(&lv->lv_lock);
 
 	return copy_to_user(arg, &lv_rate_req,
 			    sizeof(lv_rate_req)) ? -EFAULT : 0;
@@ -1206,7 +1228,8 @@ static int lvm_user_bmap(struct inode *inode, struct lv_bmap *user_result)
 	bh.b_dev = bh.b_rdev = inode->i_rdev;
 	bh.b_size = lvm_get_blksize(bh.b_dev);
 	bh.b_rsector = block * (bh.b_size >> 9);
-	if ((err=lvm_map(&bh, READ)) < 0)  {
+	bh.b_end_io = NULL;
+	if ((err = lvm_map(&bh, READ)) < 0)  {
 		printk("lvm map failed: %d\n", err);
 		return -EINVAL;
 	}
@@ -1411,6 +1434,7 @@ static int lvm_map(struct buffer_head *bh, int rw)
 
  bad:
 #if LINUX_VERSION_CODE >= KERNEL_VERSION ( 2, 4, 0)
+	if (bh->b_end_io)
 	buffer_IO_error(bh);
 #endif
 	up_read(&lv->lv_lock);
@@ -1588,23 +1612,31 @@ static int lvm_do_le_remap(vg_t *vg_ptr, void *arg)
 
 	for (l = 0; l < vg_ptr->lv_max; l++) {
 		lv_ptr = vg_ptr->lv[l];
-		if (lv_ptr != NULL &&
-		    strcmp(lv_ptr->lv_name,
-			       le_remap_req.lv_name) == 0) {
+
+		if (!lv_ptr)
+			continue;
+
+		if (strcmp(lv_ptr->lv_name, le_remap_req.lv_name) == 0) {
+			down_read(&lv_ptr->lv_lock);
 			for (le = 0; le < lv_ptr->lv_allocated_le; le++) {
 				if (lv_ptr->lv_current_pe[le].dev ==
 				    le_remap_req.old_dev &&
 				    lv_ptr->lv_current_pe[le].pe ==
 				    le_remap_req.old_pe) {
+					up_read(&lv_ptr->lv_lock);
+					down_write(&lv_ptr->lv_lock);
 					lv_ptr->lv_current_pe[le].dev =
 					    le_remap_req.new_dev;
 					lv_ptr->lv_current_pe[le].pe =
 					    le_remap_req.new_pe;
+					up_write(&lv_ptr->lv_lock);
+					down_read(&lv_ptr->lv_lock);
 
 					__update_hardsectsize(lv_ptr);
 					return 0;
 				}
 			}
+			up_read(&lv_ptr->lv_lock);
 			return -EINVAL;
 		}
 	}
@@ -1829,14 +1861,16 @@ static int lvm_do_vg_rename(vg_t *vg_ptr, void *arg)
 	for ( l = 0; l < vg_ptr->lv_max; l++)
 	{
 		if ((lv_ptr = vg_ptr->lv[l]) == NULL) continue;
+		memset (lv_ptr->vg_name, 0, sizeof (*vg_name));
 		strncpy(lv_ptr->vg_name, vg_name, sizeof ( vg_name));
 		ptr = strrchr(lv_ptr->lv_name, '/');
-		if (ptr == NULL) ptr = lv_ptr->lv_name;
+		ptr = ptr ? ptr + 1 : lv_ptr->lv_name;
 		strncpy(lv_name, ptr, sizeof ( lv_name));
 		len = sizeof(LVM_DIR_PREFIX);
 		strcpy(lv_ptr->lv_name, LVM_DIR_PREFIX);
 		strncat(lv_ptr->lv_name, vg_name, NAME_LEN - len);
-		len += strlen ( vg_name);
+		strcat (lv_ptr->lv_name, "/");
+		len += strlen(vg_name) + 1;
 		strncat(lv_ptr->lv_name, lv_name, NAME_LEN - len);
 	}
 	for ( p = 0; p < vg_ptr->pv_max; p++)
@@ -1847,6 +1881,27 @@ static int lvm_do_vg_rename(vg_t *vg_ptr, void *arg)
 
 	lvm_fs_create_vg(vg_ptr);
 
+	/* Need to add PV entries */
+	for ( p = 0; p < vg_ptr->pv_act; p++) {
+		pv_t *pv_ptr = vg_ptr->pv[p];
+
+		if (pv_ptr)
+			lvm_fs_create_pv(vg_ptr, pv_ptr);
+	}
+
+	/* Need to add LV entries */
+        for ( l = 0; l < vg_ptr->lv_max; l++) {
+		lv_t *lv_ptr = vg_ptr->lv[l];
+
+		if (!lv_ptr)
+			continue;
+
+#if LINUX_VERSION_CODE > KERNEL_VERSION ( 2, 3, 46)
+		lvm_gendisk.part[MINOR(lv_ptr->lv_dev)].de =
+#endif
+	    		lvm_fs_create_lv(vg_ptr, lv_ptr);
+	}
+
 	return 0;
 } /* lvm_do_vg_rename */
 
@@ -1921,6 +1976,9 @@ static int lvm_do_pv_create(pv_t *pvp, vg_t *vg_ptr, ulong p) {
 	pv_t *pv;
 	int err;
 
+	if (!vg_ptr)
+		return -ENXIO;
+
 	pv = kmalloc(sizeof(pv_t),GFP_KERNEL);
 	if (pv == NULL) {
 		printk(KERN_CRIT
@@ -2306,6 +2364,9 @@ static int lvm_do_lv_remove(int minor, char *lv_name, int l)
 	vg_t *vg_ptr = vg[VG_CHR(minor)];
 	lv_t *lv_ptr;
 
+	if (!vg_ptr)
+		return -ENXIO;
+
 	if (l == -1) {
 		for (l = 0; l < vg_ptr->lv_max; l++) {
 			if (vg_ptr->lv[l] != NULL &&
@@ -2536,6 +2597,9 @@ static int lvm_do_lv_extend_reduce(int minor, char *lv_name, lv_t *new_lv)
 	lv_t *old_lv;
 	pe_t *pe;
 
+	if (!vg_ptr)
+		return -ENXIO;
+
 	if ((pe = new_lv->lv_current_pe) == NULL)
 		return -EINVAL;
 
@@ -2810,6 +2874,9 @@ static int lvm_do_lv_rename(vg_t *vg_ptr, lv_req_t *lv_req, lv_t *lv)
 	int ret = 0;
 	lv_t *lv_ptr = NULL;
 
+	if (!vg_ptr)
+		return -ENXIO;
+
 	for (l = 0; l < vg_ptr->lv_max; l++)
 	{
 		if ( (lv_ptr = vg_ptr->lv[l]) == NULL) continue;
