@@ -1,12 +1,13 @@
 /*
  * tools/e2fsadm.c
  *
- * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Sistina Software
+ * Copyright (C) 1997 - 2002  Heinz Mauelshagen, Sistina Software
  *
  * June,September 1998
  * February,July,October 1999
  * January,February 2000
  * January,February 2001
+ * February 2002
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -47,6 +48,7 @@
  *               - check /proc/mounts first, don't run e2fsck for ext2resize
  *               - swap bytes on ext2 superblock values for big endian systems
  *    06/04/2001 - Tidy usage message and command-line options.
+ *    14/02/2002 - use lvm_check_number_ll() for (long long) numbers
  *
  */
 
@@ -118,7 +120,8 @@ int main ( int argc, char **argv) {
    int size_rest = 0;
    int size_equal = FALSE;
    int size_invalid = FALSE;
-   long size = 0;
+   long stripepesize = 0;
+   long long size = 0;
    char *cmd_fsck = "e2fsck";
    /* FIXME check "resize.<fstype>" and "oresize.<fstype>" instead */
    char *cmd_resize = NULL;
@@ -317,7 +320,8 @@ have_mntent:
       size_ptr++;
    }
 
-   if ( ( size = lvm_check_number ( size_ptr, opt_L > 0 ? TRUE : FALSE)) < 0) {
+   if ( ( size = lvm_check_number_ll ( size_ptr,
+                                       opt_L > 0 ? TRUE : FALSE)) < 0) {
       fprintf ( stderr, "%s -- invalid filesystem size %c%s\n", cmd,
                 sign > 0 ? '+' : '-', size_ptr);
       usage ( LVM_EE2FSADM_FSSIZE);
@@ -373,22 +377,32 @@ have_mntent:
    }
 
    if ( opt_v > 0) printf ( "%s -- checking size modulo PE size\n", cmd);
-   if ( size % vg->pe_size != 0) {
-      printf ( "%s -- correcting size %s to physical extent boundary ",
-               cmd, ( dummy = lvm_show_size ( size / 2, SHORT)));
-      free ( dummy);
-      size += vg->pe_size - ( size % vg->pe_size);
+   size_rest = size % vg->pe_size;
+   if ( size_rest > 0) {
+      printf ( "%s -- rounding size up to physical extent boundary\n", cmd);
+      size += vg->pe_size - size_rest;
       if ( size > 0 && sign == -1) size -= vg->pe_size;
-      printf ( "%s\n", ( dummy = lvm_show_size ( size / 2, SHORT)));
-      free ( dummy); dummy = NULL;
    }
 
-   size_rest = size % ( vg->lv[l]->lv_stripes * vg->pe_size);
+   /* So that -L2t can be given */
+   if ( size > LVM_LV_SIZE_2TB ( vg)) size -= vg->pe_size;
+
+   stripepesize = vg->lv[l]->lv_stripes * vg->pe_size;
+   size_rest = size % stripepesize;
    if ( size_rest != 0) {
-      printf ( "%s -- rounding size %ld KB to stripe boundary size ",
-               cmd, size / 2);
-      size = size - size_rest + ( vg->lv[l]->lv_stripes * vg->pe_size);
-      printf ( "%ld KB\n", size / 2);
+      printf ( "%s -- rounding up size to stripe boundary size\n", cmd);
+      size = size - size_rest + stripepesize;
+      if ( size > LVM_LV_SIZE_2TB ( vg)) size -= stripepesize;
+      if ( size == 0) size = stripepesize;
+   }
+
+   if ( opt_v > 0)
+      printf ( "%s -- checking logical volume maximum size\n", cmd);
+   if ( size > LVM_LV_SIZE_2TB ( vg)) {
+      fprintf ( stderr, "%s -- size %Ld KB is larger than maximum VGDA "
+                        "kernel size of %Ld KB\n",
+                        cmd, size / 2, LVM_LV_SIZE_2TB ( vg) / 2);
+      usage ( LVM_ELVCREATE_LV_SIZE);
    }
 
    if ( ( super = lvm_is_ext2_fs ( lv_name)) == NULL)
@@ -410,7 +424,9 @@ have_mntent:
          return LVM_EE2FSADM_FSCK_PATH;
       }
       if ( opt_t == 0) {
-         sprintf ( command, "%s -f %s", cmd_ptr, lv_name);
+         memset ( command, 0, sizeof ( command));
+         snprintf ( command, sizeof ( command) - 1,
+                             "%s -f %s", cmd_ptr, lv_name);
          if ( system ( command) != 0) return  LVM_EE2FSADM_FSCK_RUN;
       }
    }
@@ -422,8 +438,10 @@ have_mntent:
          lvm_path_error ( cmd_extend, "extend logical volume");
          return LVM_EE2FSADM_LV_EXTEND_PATH;
       }
-      sprintf ( command, "%s -l %lu %s",
-                         cmd_ptr, size / vg->pe_size, lv_name);
+      memset ( command, 0, sizeof ( command));
+      snprintf ( command, sizeof ( command) - 1,
+                          "%s -l %Ld %s",
+                          cmd_ptr, size / vg->pe_size, lv_name);
       if ( opt_t == 0) {
          if ( system ( command) != 0) return LVM_EE2FSADM_LV_EXTEND_RUN;
          if ( ( ret = lvm_tab_lv_read_by_name ( vg_name, lv_name, &lv)) < 0) {
@@ -445,9 +463,11 @@ have_mntent:
                          cmd_resize : cmd_resize_default, "resize");
         return LVM_EE2FSADM_RESIZE_PATH;
       }
-      sprintf ( command, "%s %s %s %lu",
-                         cmd_ptr, cmd_resize_opts,
-                         lv_name, size >> ( super->s_log_block_size + 1));
+      memset ( command, 0, sizeof ( command));
+      snprintf ( command, sizeof ( command) - 1,
+                          "%s %s %s %Ld",
+                          cmd_ptr, cmd_resize_opts,
+                          lv_name, size >> ( super->s_log_block_size + 1));
       if ( opt_t == 0) {
          if ( system ( command) != 0) return LVM_EE2FSADM_RESIZE_RUN;
          if ( ( super = lvm_is_ext2_fs ( lv_name)) == NULL)
@@ -469,9 +489,11 @@ have_mntent:
                           cmd_resize : cmd_resize_default, "resize");
          return LVM_EE2FSADM_RESIZE_PATH;
       }
-      sprintf ( command, "%s %s %s %lu",
-                         cmd_ptr, cmd_resize_opts,
-                         lv_name, size >> ( super->s_log_block_size + 1));
+      memset ( command, 0, sizeof ( command));
+      snprintf ( command, sizeof ( command) - 1,
+                          "%s %s %s %Ld",
+                          cmd_ptr, cmd_resize_opts,
+                          lv_name, size >> ( super->s_log_block_size + 1));
       if ( opt_t == 0) {
          if ( system ( command) != 0) return LVM_EE2FSADM_RESIZE_RUN;
          if ( ( super = lvm_is_ext2_fs ( lv_name)) == NULL)
@@ -488,8 +510,10 @@ have_mntent:
          lvm_path_error ( cmd_reduce, "reduce logical volume");
          return LVM_EE2FSADM_LV_REDUCE_PATH;
       }
-      sprintf ( command, "%s -f -l %lu %s",
-                         cmd_ptr, size / vg->pe_size, lv_name);
+      memset ( command, 0, sizeof ( command));
+      snprintf ( command, sizeof ( command) - 1,
+                          "%s -f -l %Ld %s",
+                          cmd_ptr, size / vg->pe_size, lv_name);
       if ( opt_t == 0) {
          if ( system ( command) != 0) return LVM_EE2FSADM_LV_REDUCE_RUN;
       }
@@ -512,7 +536,9 @@ char *lvm_find_command ( char *cmd_to_find) {
    if ( cmd_to_find == NULL || strlen ( cmd_to_find) == 0) return ret;
 
    while ( bin_dirs[i] != NULL) {
-      sprintf ( command, "%s/%s", bin_dirs[i], cmd_to_find);
+      memset ( command, 0, sizeof ( command));
+      snprintf ( command, sizeof ( command) - 1,
+                          "%s/%s", bin_dirs[i], cmd_to_find);
       if ( stat ( command, &stat_b) == 0) {
          ret = command;
          break;
