@@ -93,7 +93,7 @@
 #include <time.h>
 #include <limits.h>
 #ifdef _G_LSEEK64
-int lseek64 ( unsigned int, unsigned long long, unsigned int);
+loff_t lseek64 ( unsigned int, unsigned long long, unsigned int);
 #define llseek lseek64
 #else
 int llseek ( unsigned int, unsigned long long, unsigned int);
@@ -810,7 +810,8 @@ char **lvm_tab_vg_check_exist_all_vg ( void);
 #define LVM_EVG_WRITE_WRITE                                                 404
 #define LVM_ELV_PV_CREATE_NAME_FROM_KDEV_T                                  405
 #define LVM_EPV_FLUSH_STAT                                                  406
-#define LVM_MAX_ERROR							    407
+#define LVM_EPV_MOVE_PES_SNAPSHOT                                           407
+#define LVM_MAX_ERROR							    408
 
 
 /*
