@@ -1,10 +1,11 @@
 /*
  * tools/lib/lv_release.c
  *
- * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Sistina Software
+ * Copyright (C) 1997 - 2001  Heinz Mauelshagen, Sistina Software
  *
  * March 1997
  * January 2000
+ * January,April 2001
  *
  *
  * This LVM library is free software; you can redistribute it and/or
@@ -28,6 +29,9 @@
  * Changelog
  *
  *    31/01/2000 - use debug_enter()/debug_leave()
+ *    08/01/2000 - reset the original snapshot in case the last
+ *                 snapshot is released
+ *    09/04/2001 - cleaned up debug output (Andreas Dilger)
  *
  */
 
@@ -41,9 +45,7 @@ int lv_release ( vg_t *vg, char *lv_name) {
    uint pe_released = 0;
    int ret = 0;
 
-#ifdef DEBUG
    debug_enter ( "lv_release -- CALLED with %s\n", lv_name);
-#endif
 
    if ( vg == NULL || lv_check_name ( lv_name) < 0) {
       ret = -LVM_EPARAM;
@@ -54,44 +56,31 @@ int lv_release ( vg_t *vg, char *lv_name) {
    for ( l = 0; l < vg->lv_max; l++)
       if ( vg->lv[l] != NULL &&
            strcmp ( vg->lv[l]->lv_name, lv_name) == 0) break;
-#ifdef DEBUG
    debug ( "lv_release -- after search for %s\n", lv_name);
-#endif
-
    if ( l == vg->lv_max) {
-#ifdef DEBUG
       debug ( "lv_release -- %s NOT found\n", lv_name);
-#endif
       ret = -LVM_ELV_RELEASE_LV_NUM;
       goto lv_release_end;
    }
-#ifdef DEBUG
    debug ( "lv_release -- %s found\n", lv_name);
-#endif
-
-/*
-   for ( ll = 0; ll < vg->lv_max; ll++)
-      if ( vg->lv[ll] != NULL &&
-           vg->lv[ll]->lv_dev = vg->lv[l]->lv_snapshot_MINOR)
-         vg->lv[l]->lv_access &= ~LV_SNAPSHOT_ORG;
-*/
+   /* if it was a snapshot we optionally need to
+      update the snapshot origin */
+   if ( vg->lv[l]->lv_access & LV_SNAPSHOT &&
+       vg->lv[l]->lv_snapshot_next == NULL &&
+       vg->lv[l]->lv_snapshot_org->lv_snapshot_next == vg->lv[l]) {
+      vg->lv[l]->lv_snapshot_org->lv_access &= ~LV_SNAPSHOT_ORG;
+   }
 
    lv_num = l + 1;
-
-#ifdef DEBUG
    debug ( "lv_release -- l: %d  lv_num: %d  pv_cur: %lu\n",
             l, lv_num, vg->pv_cur);
-#endif
-
    /* walk through physical volumes freeing physical extents */
    p = 0;
    for ( p = 0; p < vg->pv_cur; p++) {
       pe_released = 0;
-#ifdef DEBUG
       debug ( "lv_release -- vg->pv[%d]: %X\n", p, ( uint) vg->pv[p]);
       debug ( "lv_release -- vg->pv[%d]->pe: %X\n", p,
                ( uint) vg->pv[p]->pe);
-#endif
       /* walk through physical extents */
       for ( pe = 0; pe < vg->pv[p]->pe_total; pe++) {
          if ( vg->pv[p]->pe[pe].lv_num == lv_num) {
@@ -109,8 +98,7 @@ int lv_release ( vg_t *vg, char *lv_name) {
    vg->lv[l] = NULL;
 
 lv_release_end:
-#ifdef DEBUG
+
    debug_leave ( "lv_release -- LEAVING with ret: %d\n", ret);
-#endif
    return ret;
 }
