@@ -61,10 +61,8 @@ int lv_setup_for_create ( char *vg_name, vg_t **vg, char *lv_name,
    vg_t *vg_this;
    pe_t *pe_p = NULL;
 
-#ifdef DEBUG
    debug_enter ( "lv_setup_for_create -- CALLED  %s  %d  %s\n",
                  vg_name, size, lv_name);
-#endif
 
    if ( vg_name == NULL || vg == NULL || lv_name == NULL ||
         strchr ( lv_name, '/') != NULL ||
@@ -80,6 +78,7 @@ int lv_setup_for_create ( char *vg_name, vg_t **vg, char *lv_name,
    }
 
    vg_this = *vg;
+
    if ( stripesize > vg_this->pe_size) {
       ret = -LVM_ELV_SETUP_FOR_CREATE_STRIPESIZE;
       goto lv_setup_for_create_end;
@@ -97,11 +96,7 @@ int lv_setup_for_create ( char *vg_name, vg_t **vg, char *lv_name,
       goto lv_setup_for_create_end;
    }
    *lv_index = l;
-
-#ifdef DEBUG
    debug ( "lv_setup_for_create -- l: %d\n", l);
-#endif
-
    /* and create it */
    if ( ( vg_this->lv[l] = malloc ( sizeof ( lv_t))) == NULL) {
       fprintf ( stderr, "malloc error in %s [line %d]\n",
@@ -109,24 +104,20 @@ int lv_setup_for_create ( char *vg_name, vg_t **vg, char *lv_name,
       ret = -LVM_ELV_SETUP_FOR_CREATE_MALLOC;
       goto lv_setup_for_create_end;
    }
-#ifdef DEBUG
    debug ( "lv_setup_for_create -- vg_this->lv[%d]: %X\n",
             l, ( uint) vg_this->lv[l]);
-#endif
    if ( strlen ( lv_name) == 0) {
-      sprintf ( lv_name, LVM_DIR_PREFIX "%s/lvol%d%c", vg_name, l + 1, 0);
+      sprintf ( lv_name, LVM_DIR_PREFIX "%s/lvol%d", vg_name, l + 1);
    } else {
       strncpy ( lv_name_this, lv_name,
                 sizeof ( lv_name_this) -
                 strlen ( vg_name) -
                 strlen ( LVM_DIR_PREFIX) - 2);
-      sprintf ( lv_name, LVM_DIR_PREFIX "%s/%s%c", vg_name, lv_name_this, 0);
+      sprintf ( lv_name, LVM_DIR_PREFIX "%s/%s", vg_name, lv_name_this);
    }
    lv_pe.lv_num = l + 1;
    lv_pe.le_num = 0;
-#ifdef DEBUG
    debug ( "lv_setup_for_create -- creating %s  %d\n", lv_name, lv_pe.lv_num);
-#endif
    strcpy ( vg_this->lv[l]->vg_name, vg_name);
    strcpy ( vg_this->lv[l]->lv_name, lv_name);
    vg_this->lv[l]->lv_access = permission;
@@ -142,9 +133,7 @@ int lv_setup_for_create ( char *vg_name, vg_t **vg, char *lv_name,
    vg_this->lv[l]->lv_recovery = 0;
    vg_this->lv[l]->lv_schedule = 0;
    vg_this->lv[l]->lv_size = size / vg_this->pe_size * vg_this->pe_size;
-#ifdef DEBUG
    debug ( "lv_setup_for_create -- lv_size: %lu\n", vg_this->lv[l]->lv_size);
-#endif
    vg_this->lv[l]->lv_current_le = size / vg_this->pe_size;
    if ( vg_this->lv[l]->lv_current_le >
         vg_this->pe_total - vg_this->pe_allocated) {
@@ -178,7 +167,6 @@ int lv_setup_for_create ( char *vg_name, vg_t **vg, char *lv_name,
    }
 
    /* walk through physical volumes searching for physical extents */
-   stripes = vg_this->lv[l]->lv_stripes;
    pe = vg_this->lv[l]->lv_allocated_le;
 
    p = 0;
@@ -201,10 +189,8 @@ int lv_setup_for_create ( char *vg_name, vg_t **vg, char *lv_name,
          }
 
          if ( ! ( vg_this->pv[p]->pv_allocatable & PV_ALLOCATABLE)) {
-#ifdef DEBUG
             debug ( "lv_setup_for_create -- %s NOT allocatable\n",
                       vg_this->pv[p]->pv_name);
-#endif
             p++;
             continue;
          }
@@ -218,15 +204,11 @@ int lv_setup_for_create ( char *vg_name, vg_t **vg, char *lv_name,
             goto lv_setup_for_create_end;
          }
          if ( pe < pe_last) vg_this->pv[p]->lv_cur++;
-#ifdef DEBUG
          debug ( "lv_setup_for_create -- pv_reserve_pe returned: %d   "
                   "pe_last: %d  pe: %d\n", ret, pe_last, pe);
-#endif
          p++;
       }
-#ifdef DEBUG
       debug ( "lv_setup_for_create -- pe: %d\n", pe);
-#endif
       if ( pe != 0) {
          ret = -LVM_ESIZE;
          goto lv_setup_for_create_end;
@@ -256,10 +238,8 @@ int lv_setup_for_create ( char *vg_name, vg_t **vg, char *lv_name,
          }
 
          if ( ! ( vg_this->pv[p]->pv_allocatable & PV_ALLOCATABLE)) {
-#ifdef DEBUG
             debug ( "lv_setup_for_create -- %s NOT allocatable\n",
                       vg_this->pv[p]->pv_name);
-#endif
             p++;
             continue;
          }
@@ -268,18 +248,14 @@ int lv_setup_for_create ( char *vg_name, vg_t **vg, char *lv_name,
             pe_this = pe_last;
             pe_p = &vg_this->lv[l]->lv_current_pe[vg_this->lv[l]->
                    lv_allocated_le-pe];
-#ifdef DEBUG
             debug ( "lv_setup_for_create -- pe_this: %d\n", pe_this);
-#endif
             if ( ( ret = pv_reserve_pe ( vg_this->pv[p],
                                          &lv_pe, &pe_this,
                                          pe_p, allocation, TRUE)) < 0) {
                ret = -LVM_ESIZE;
                goto lv_setup_for_create_end;
             }
-#ifdef DEBUG
             debug ( "lv_setup_for_create -- pe_this: %d\n", pe_this);
-#endif
             if ( pe_this == 0) {
                vg_this->pv[p]->lv_cur++;
                pe -= pe_last;
@@ -301,8 +277,7 @@ int lv_setup_for_create ( char *vg_name, vg_t **vg, char *lv_name,
    vg_this->pe_allocated += vg_this->lv[l]->lv_allocated_le;
 
 lv_setup_for_create_end:
-#ifdef DEBUG
+
    debug_leave ( "lv_setup_for_create -- LEAVING with ret: %d\n", ret);
-#endif
    return ret;
 }
