@@ -1,11 +1,12 @@
 /*
  * tools/lib/lvm_interrupt.c
  *
- * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Sistina Software
+ * Copyright (C) 1997 - 2001  Heinz Mauelshagen, Sistina Software
  *
  * March-May 1997
  * January 1998
  * January 2000
+ * April 2001
  *
  *
  * This LVM library is free software; you can redistribute it and/or
@@ -30,6 +31,7 @@
  *
  *    17/01/1998 - ported to Linux 2.1.79
  *    31/01/2000 - use debug_enter()/debug_leave()
+ *    09/04/2001 - cleaned up debug output (Andreas Dilger)
  *
  */
 
@@ -49,14 +51,11 @@ void lvm_interrupt ( void) {
 #ifndef	LVM_DONT_INTERRUPT
    int s;
 
-#ifdef DEBUG
    debug_enter ( "lvm_interrupt -- CALLED\n");
-#endif
 
    for ( s = 0; s < NSIG; s++) signal ( s, SIG_DFL);
-#ifdef DEBUG
+
    debug_leave ( "lvm_interrupt -- LEAVING\n");
-#endif
 #endif
    return;
 }
@@ -65,17 +64,14 @@ void lvm_dont_interrupt ( int sig) {
 #ifndef	LVM_DONT_INTERRUPT
    int s;
 
-#ifdef DEBUG
    debug_enter ( "lvm_dont_interrupt -- CALLED\n");
-#endif
+
    for ( s = 0; s < NSIG; s++) {
       if ( s != SIGKILL)
          signal ( s, SIG_IGN);
    }
 
-#ifdef DEBUG
-   debug_leave ( "lvm_interrupt -- LEAVING\n");
-#endif
+   debug_leave ( "lvm_dont_interrupt -- LEAVING\n");
 #endif
    return;
 }
