@@ -1,7 +1,7 @@
 /*
  * tools/lib/lvm_tab_write.c
  *
- * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Sistina Software
  *
  * Oktober-November 1997
  * May,June 1998
@@ -48,9 +48,7 @@ int lvm_tab_write ( char *data, int size) {
    int out = -1;
    int ret = 0;
 
-#ifdef DEBUG
    debug_enter ( "lvm_tab_write -- CALLED\n");
-#endif
 
    if ( data == NULL || size == 0) ret = -LVM_EPARAM;
    else if ( ( out = open ( LVMTAB, O_WRONLY | O_CREAT | O_TRUNC, 0640)) == -1)
@@ -65,8 +63,6 @@ int lvm_tab_write ( char *data, int size) {
       close ( out);
    }
 
-#ifdef DEBUG
    debug_leave ( "lvm_tab_write -- LEAVING with ret: %d\n", ret);
-#endif
    return ret;
 }
