@@ -39,9 +39,7 @@ int pv_check_free ( pv_t *pv, ulong pe_count, ulong *pe_start) {
    int ret = FALSE;
    ulong pe = 0;
 
-#ifdef DEBUG
    debug_enter ( "pv_check_free -- CALLED\n");
-#endif
 
    if ( pv == NULL || pe_count < 1)
       ret = -LVM_EPARAM;
@@ -57,9 +55,7 @@ int pv_check_free ( pv_t *pv, ulong pe_count, ulong *pe_start) {
       }
    }
 
-#ifdef DEBUG
    debug_leave ( "pv_check_free -- LEAVING with ret: %d\n", ret);
-#endif
    return ret;
 }
 
@@ -70,9 +66,7 @@ int pv_check_free_contiguous ( pv_t *pv, ulong pe_count, ulong *pe_start) {
    ulong this_pe_count = 0;
    ulong this_pe_start = 0;
 
-#ifdef DEBUG
    debug_enter ( "pv_check_free_contiguous -- CALLED\n");
-#endif
 
    if ( pv == NULL || pe_count < 1) ret = -LVM_EPARAM;
    else {
@@ -92,8 +86,6 @@ int pv_check_free_contiguous ( pv_t *pv, ulong pe_count, ulong *pe_start) {
       }
    }
 
-#ifdef DEBUG
    debug_leave ( "pv_check_free_contiguous -- LEAVING with ret: %d\n", ret);
-#endif
    return ret;
 }
