@@ -1,7 +1,7 @@
 /*
  * tools/lib/pv_check_in_vg.c
  *
- * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Sistina Software
  *
  * May 1997
  * June 1998
@@ -38,17 +38,13 @@
 int pv_check_in_vg ( vg_t *vg, char *pv_name) {
    int ret = FALSE;
 
-#ifdef DEBUG
    debug_enter ( "pv_check_in_vg -- CALLED\n");
-#endif
 
    if ( vg == NULL || pv_name == NULL ||
         vg_check_name ( vg->vg_name) < 0 ||
         pv_check_name ( pv_name) < 0) ret = -LVM_EPARAM;
    else if ( pv_get_index_by_name ( vg, pv_name) > -1 ) ret = TRUE;
-   
-#ifdef DEBUG
+
    debug_leave ( "pv_check_in_vg -- LEAVING with ret: %d\n", ret);
-#endif
    return ret;
 }
