@@ -47,10 +47,8 @@ char *pv_create_name_from_kdev_t ( kdev_t dev) {
    struct stat stat_b;
    dir_cache_t *dir_cache = NULL;
 
-#ifdef DEBUG
    debug_enter ( "pv_create_name_from_kdev_t -- CALLED with %d:%d\n",
                  MAJOR ( dev), MINOR ( dev));
-#endif
 
    stat_b.st_rdev = dev;
    if ( lvm_check_dev ( &stat_b, FALSE) == FALSE) ret = NULL;
@@ -67,9 +65,7 @@ char *pv_create_name_from_kdev_t ( kdev_t dev) {
       } else ret = NULL;
    }
 
-#ifdef DEBUG
    debug_leave ( "pv_create_name_from_kdev_t -- LEAVING with dev_name: %s\n",
                  ret);
-#endif
    return ret;
 }
