@@ -1,7 +1,7 @@
 /*
  * tools/lib/pv_get_index.c
  *
- * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Sistina Software
  *
  * May 1997
  * January 2000
@@ -39,11 +39,9 @@ int pv_get_index_by_kdev_t ( vg_t *vg, kdev_t dev) {
    int p;
    int ret = -1;
 
-#ifdef DEBUG
    debug_enter ( "pv_get_index_by_kdev_t -- CALLED for VG \"%s\" "
                  "and %02d:%02d\n",
                  vg->vg_name, MAJOR ( dev), MINOR ( dev));
-#endif
 
    if ( vg != NULL && vg_check_name ( vg->vg_name) == 0) {
       for ( p = 0; p < vg->pv_max; p++) {
@@ -54,9 +52,7 @@ int pv_get_index_by_kdev_t ( vg_t *vg, kdev_t dev) {
       }
    } else ret = -LVM_EPARAM;
 
-#ifdef DEBUG
    debug_leave ( "pv_get_index_by_kdev_t -- LEAVING with ret: %d\n", ret);
-#endif
    return ret;
 }
 
@@ -65,9 +61,7 @@ int pv_get_index_by_name ( vg_t *vg, char *pv_name) {
    int p;
    int ret = -1;
 
-#ifdef DEBUG
    debug_enter ( "pv_get_index_by_name -- CALLED\n");
-#endif
 
    if ( vg == NULL || vg_check_name ( vg->vg_name) < 0 ||
         pv_name == NULL || pv_check_name ( pv_name) < 0) ret = -LVM_EPARAM;
@@ -80,8 +74,6 @@ int pv_get_index_by_name ( vg_t *vg, char *pv_name) {
       }
    }
 
-#ifdef DEBUG
    debug_leave ( "pv_get_index_by_name -- LEAVING with ret: %d\n", ret);
-#endif
    return ret;
 }
