@@ -74,24 +74,25 @@
 extern int opt_ignore;
 int vg_active = TRUE;
 
-/* internal functions */
-ulong  _read ( int, char*, ulong, char*);
-loff_t _llseek ( int, loff_t, int);
-void   pv_move_interrupt ( int);
-int   _pe_lock ( char*, kdev_t, ulong, ushort, ushort, kdev_t);
-int   _pe_unlock ( char*);
-int   _lv_le_remap ( vg_t*, le_remap_req_t*);
-int   _pv_change ( char*, pv_t*);
-
-static int pv_move_int = FALSE;
-static lv_t *lv_this = NULL;
-
-struct pe_struct {
+typedef struct pe_struct {
    ushort p;
    long pe;
    ushort lv_num;
    ushort le_num;
-};
+} pe_struct_t;
+
+/* internal functions */
+static ulong _read ( int, char*, ulong, char*);
+loff_t       _llseek ( int, loff_t, int);
+void         pv_move_interrupt ( int);
+int         _pe_lock ( char*, kdev_t, ulong, ushort, ushort, kdev_t);
+int         _pe_unlock ( char*);
+int         _lv_le_remap ( vg_t*, le_remap_req_t*);
+int         _pv_change ( char*, pv_t*);
+static int  _belong_to_snapshot ( vg_t *vg, pe_struct_t *, int); 
+
+static int pv_move_int = FALSE;
+static lv_t *lv_this = NULL;
 
 int pe_compare(const void *one, const void *two)
 {
@@ -245,6 +246,13 @@ int pv_move_pes ( vg_t *vg, char *buffer, size_t buffer_size, char **pv_allowed,
    /* sort source PEs in ascending LV, LE order */
    qsort ( pe_src, pe_to_move, sizeof ( pe_tmp), pe_compare);
 
+   /* Check if a source extent belongs to an original
+      or snapshot logical volume */
+   if ( _belong_to_snapshot ( vg, pe_src, pe_to_move)) {
+      ret = -LVM_EPV_MOVE_PES_SNAPSHOT;
+      goto pv_move_pes_end;
+   }
+
    /* get all destination PEs */
    pe_for_dest = 0;
    for ( pa = 0; pv_allowed[pa] != NULL && pe_for_dest < pe_to_move; pa++) {
@@ -687,45 +695,58 @@ pv_move_interrupt_end:
 loff_t _llseek ( int fh, loff_t offset, int whence) {
    loff_t pos = llseek ( fh, offset, whence);
 
-   if ( pos == ( loff_t) -1) {
-      if ( opt_ignore > 0) pos = 0;
-   }
+   if ( opt_ignore &&
+        pos == -1)
+      pos = 0;
+
    return pos;
 }
 
 /* read with fallback to SECTOR_SIZEed io in case of an error */
-ulong _read ( int in, char *buffer, ulong to_read, char *pv_name) {
+static ulong _read ( int in, char *buffer, ulong to_read, char *pv_name) {
    ulong ret;
    ulong total_read = 0;
-   loff_t pos = _llseek ( in, 0, SEEK_CUR);
+   loff_t pos = _llseek ( in, 0, SEEK_CUR); /* get actual position */
+
+   /* if we can't even get our actual position */
+   if ( pos <= 0) {
+      if ( opt_ignore == 0)
+         return 0;
+      else
+         return to_read;
+   }
 
    memset ( buffer, 0, to_read);
 
-   /* if we can't even seek */
-   if ( pos <= 0) {
-      if ( opt_ignore == 0) return 0;
-      return to_read;
-   }
+   ret = read ( in, buffer, to_read);
+   /* If we could read it fine, return it :-) */
+   if ( ret == to_read)
+      return ret;
 
-   if ( ( ret = read ( in, buffer, to_read)) != to_read) {
-      /* in case we shouldn't ignore, that's an error */
-      if ( opt_ignore == 0) return ret;
-
-      /* redo with SECTOR_SIZE reads */
+   /* in case we should ignore and couldn't read the amount
+      we wanted, we start to read single sectors */
+   if ( opt_ignore > 0) {
       memset ( buffer, 0, to_read);
-      if ( _llseek ( in, pos, SEEK_SET) <= 0) return to_read;
+
       while ( total_read < to_read) {
-         if ( ( ret = read ( in, buffer, SECTOR_SIZE)) != SECTOR_SIZE) {
-            fprintf ( stderr, "%s -- ERROR \"%s\" reading sector %Ld from "
+         /* If we can't position (for eg. on a test device),
+            return buffer of zeroed bytes */
+         if ( _llseek ( in, pos, SEEK_SET) <= 0)
+            return to_read;
+
+         ret = read ( in, buffer, SECTOR_SIZE);
+         if ( ret != SECTOR_SIZE) {
+            fprintf ( stderr, "%s -- ERROR \"%s\" reading sector %lld from "
                               "\"%s\"\n",
                               cmd, strerror ( errno),
                               pos / SECTOR_SIZE, pv_name);
-          }
-          pos += SECTOR_SIZE;
-          total_read += SECTOR_SIZE;
-          if ( _llseek ( in, pos, SEEK_SET) <= 0) return total_read;
+         }
+         pos += SECTOR_SIZE;
+         total_read += SECTOR_SIZE;
       }
+      ret = total_read;
    }
+
    return ret;
 } /* _read() */
 
@@ -761,3 +782,40 @@ int _pv_change ( char *vg_name, pv_t *pv) {
    else
       return 0;
 }
+
+
+lv_t *_get_lv_for_offset ( vg_t *vg, pv_t *pv, ulong offset) {
+   int l, le;
+   lv_t *lv;
+
+   for ( l = 0; l < vg->lv_max; l++) {
+      lv = vg->lv[l];
+      if ( lv == NULL) continue;
+      for ( le = 0; le < lv->lv_allocated_le; le++) {
+         if ( lv->lv_current_pe[le].dev != pv->pv_dev)
+            continue;
+         if ( lv->lv_current_pe[le].pe == offset)
+            return lv;
+      }
+   }
+
+   return NULL;
+}
+
+static int _belong_to_snapshot ( vg_t *vg, pe_struct_t *pe_src, int pes) {
+   int pe;
+
+   for ( pe = 0; pe < pes; pe++) {
+      pv_t *pv = vg->pv[pe_src[pe].p];
+      lv_t *lv;
+
+      lv = _get_lv_for_offset ( vg, pv, get_pe_offset ( pe_src[pe].pe, pv));
+      if ( lv == NULL) continue;
+
+      if ((lv->lv_access & LV_SNAPSHOT_ORG) ||
+          (lv->lv_access & LV_SNAPSHOT))
+         return 1;
+   }
+
+   return 0;
+}
