@@ -1,11 +1,12 @@
 /*
  * tools/lib/pv_write_pe.c
  *
- * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Sistina Software
+ * Copyright (C) 1997 - 2001  Heinz Mauelshagen, Sistina Software
  *
  * March 1997
  * August-September 1998
  * January 2000
+ * June 2001
  *
  *
  * This LVM library is free software; you can redistribute it and/or
@@ -31,6 +32,7 @@
  *    06/09/1998 - implemented data layer in pv_write_pe() by using
  *                 new function pe_copy_to_disk()
  *    08/02/2000 - use debug_enter()/debug_leave()
+ *    22/06/2001 - added Andreas Dilger's PE on 4k boundary alignment enhancements
  *
  */
 
@@ -43,16 +45,14 @@ int pv_write_pe ( char *pv_name, pv_t *pv) {
    uint size = 0;
    pe_disk_t *pe_disk = NULL;
 
-#ifdef DEBUG
    debug_enter ( "pv_write_pe -- CALLED  pv->pe_total: %lu\n", pv->pe_total);
-#endif
 
    if ( pv_name == NULL ||
         pv_check_name ( pv_name) < 0 ||
         pv == NULL) ret = -LVM_EPARAM;
    else {
       size = pv->pe_total * sizeof ( pe_disk_t);
-      if ( size + pv->pe_on_disk.base > LVM_DISK_SIZE ( pv))
+      if ( size + pv->pe_on_disk.base > LVM_VGDA_SIZE ( pv))
          ret = -LVM_EPV_WRITE_PE_SIZE;;
       if ( ( pv_handle = open ( pv_name, O_WRONLY)) == -1)
          ret = -LVM_EPV_WRITE_PE_OPEN;
@@ -72,8 +72,6 @@ int pv_write_pe ( char *pv_name, pv_t *pv) {
       }
    }
 
-#ifdef DEBUG
    debug_leave ( "pv_write_pe -- LEAVING with ret: %d\n", ret);
-#endif
    return ret;
 }
