@@ -36,33 +36,27 @@
 #include <liblvm.h>
 #include <sys/utsname.h>
 
-inline int system_id_set ( char *system_id) {
+int system_id_set ( char *system_id) {
    int ret = 0;
    struct utsname uts;
 
-#ifdef DEBUG
    debug_enter ( "system_id_set -- CALLED\n");
-#endif
 
    if ( system_id != NULL) {
       if ( uname ( &uts) != 0) ret = -LVM_ESYSTEM_ID_SET_UNAME;
-      else sprintf ( system_id, "%s%lu%c", uts.nodename, time ( NULL), 0);
+      else sprintf ( system_id, "%s%lu", uts.nodename, time ( NULL));
    } else ret = -LVM_EPARAM;
 
-#ifdef DEBUG
    debug_leave ( "system_id_set -- LEAVING with ret: %d\n", ret);
-#endif
    return ret;
 }
 
 
-inline int system_id_set_exported ( char *system_id) {
+int system_id_set_exported ( char *system_id) {
    int ret = 0;
    struct utsname uts;
 
-#ifdef DEBUG
    debug_enter ( "system_id_set_exported -- CALLED\n");
-#endif
 
    if ( system_id != NULL) {
       if ( uname ( &uts) != 0) ret = -LVM_ESYSTEM_ID_SET_UNAME;
@@ -70,40 +64,30 @@ inline int system_id_set_exported ( char *system_id) {
                      EXPORTED, uts.nodename, time ( NULL), 0);
    } else ret = -LVM_EPARAM;
 
-#ifdef DEBUG
    debug_leave ( "system_id_set_exported -- LEAVING with ret: %d\n", ret);
-#endif
-
    return ret;
 }
 
 
-inline int system_id_set_imported ( char *system_id) {
+int system_id_set_imported ( char *system_id) {
    int ret = 0;
 
-#ifdef DEBUG
    debug_enter ( "system_id_set_imported -- CALLED\n");
-#endif
 
    if ( system_id != NULL) {
       sprintf ( system_id, "%s%s%c", IMPORTED,
                 &system_id[sizeof ( EXPORTED) - 1], 0);
    } else ret = -LVM_EPARAM;
 
-#ifdef DEBUG
    debug_leave ( "system_id_set_imported -- LEAVING\n");
-#endif
-
    return ret;
 }
 
 
-inline int system_id_check_exported ( char *system_id) {
+int system_id_check_exported ( char *system_id) {
    int ret = UNDEF;
 
-#ifdef DEBUG
    debug_enter ( "system_id_check_exported -- CALLED\n");
-#endif
 
    if ( system_id != NULL) {
       if ( strncmp ( system_id, EXPORTED, sizeof ( EXPORTED) - 1) == 0)
@@ -112,19 +96,15 @@ inline int system_id_check_exported ( char *system_id) {
          ret = FALSE;
    } else ret = -LVM_EPARAM;
 
-#ifdef DEBUG
    debug_leave ( "system_id_check_exported -- LEAVING with ret: %d\n", ret);
-#endif
    return ret;
 }
 
 
-inline int system_id_check_imported ( char *system_id) {
+int system_id_check_imported ( char *system_id) {
    int ret = UNDEF;
 
-#ifdef DEBUG
    debug_enter ( "system_id_check_imported -- CALLED\n");
-#endif
 
    if ( system_id != NULL) {
       if ( strncmp ( system_id, IMPORTED, sizeof ( IMPORTED) - 1) == 0)
@@ -133,8 +113,6 @@ inline int system_id_check_imported ( char *system_id) {
          ret = FALSE;
    } else ret = -LVM_EPARAM;
 
-#ifdef DEBUG
    debug_leave ( "system_id_check_imported -- LEAVING with ret: %d\n", ret);
-#endif
    return ret;
 }
