@@ -1,13 +1,14 @@
 /*
  * tools/lvcreate.c
  *
- * Copyright (C) 1997 - 2001  Heinz Mauelshagen, Sistina Software
+ * Copyright (C) 1997 - 2002  Heinz Mauelshagen, Sistina Software
  *
  * March,October,November 1997
  * January,May-July,September,December 1998
  * January,February,July,October 1999
  * February,July,September,November 2000
  * January,March,June,July 2001
+ * February 2002
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -76,6 +77,8 @@
  *               - fixed wrong snapshot chunk size message
  *    07/11/2001 - Add usage message, clean up printf [AED]
  *    25/07/2001 - lv_path->lv_name in err mesg (AGK)
+ *    07/02/2002 - fixes for > 1TB support
+ *    14/02/2002 - use lvm_check_number_ll() for (long long) numbers
  *
  */
 
@@ -155,8 +158,8 @@ int main ( int argc, char **argv) {
    int opt_v = 0;
    int opt_Z = 1;
    int ret = 0;
-   int size = 0;
    int size_rest = 0;
+   long long size = 0;
    long read_ahead = 0;
    uint allocation = 0;
    uint mirrors = 0;
@@ -285,7 +288,7 @@ int main ( int argc, char **argv) {
                usage ( LVM_EINVALID_CMD_LINE);
             }
 
-            if ( ( size = lvm_check_number ( optarg, FALSE)) > 0) {
+            if ( ( size = lvm_check_number_ll ( optarg, FALSE)) > 0) {
                opt_l++;
             } else {
                fprintf ( stderr, "%s -- ERROR option l argument \"%s\"\n",
@@ -300,7 +303,7 @@ int main ( int argc, char **argv) {
                usage ( LVM_EINVALID_CMD_LINE);
             }
 
-            if ( ( size = lvm_check_number ( optarg, TRUE)) > 0) {
+            if ( ( size = lvm_check_number_ll ( optarg, TRUE)) > 0) {
                opt_L++;
             } else {
                fprintf ( stderr, "%s -- ERROR option L argument \"%s\"\n\n",
@@ -426,7 +429,9 @@ int main ( int argc, char **argv) {
          usage ( LVM_ELVCREATE_LV_NAME);
       }
       vg_name = vg_name_of_lv ( lv_name);
-      sprintf ( lv_path, LVM_DIR_PREFIX "%s/%s", vg_name, lv_snapshot_name);
+      memset ( lv_path, 0, sizeof ( lv_path));
+      snprintf ( lv_path, sizeof ( lv_path) - 1,
+                          LVM_DIR_PREFIX "%s/%s", vg_name, lv_snapshot_name);
 
       if ( opt_v > 0)
          printf ( "%s -- checking snapshot logical volume name \"%s\"\n",
@@ -482,7 +487,9 @@ int main ( int argc, char **argv) {
                            cmd, sizeof ( lv_path));
          return LVM_ELVCREATE_LV_CHECK_NAME;
       }
-      sprintf ( lv_path, LVM_DIR_PREFIX "%s/%s", vg_name, lv_name);
+      memset ( lv_path, 0, sizeof ( lv_path));
+      snprintf ( lv_path, sizeof ( lv_path) - 1,
+                          LVM_DIR_PREFIX "%s/%s", vg_name, lv_name);
 
       if ( opt_v > 0) printf ( "%s -- checking logical volume path \"%s\"\n",
                                cmd, lv_path);
@@ -556,7 +563,7 @@ int main ( int argc, char **argv) {
 
    if ( opt_v > 0)
       printf ( "%s -- getting volume group status from VGDA in kernel\n", cmd);
-   if ( ( ret = vg_status_with_pv_and_lv ( vg_name, &vg_core)) != 0) {
+   if ( ( ret = vg_status_with_pv_and_lv ( vg_name, &vg_core)) < 0) {
       fprintf ( stderr, "%s -- ERROR \"%s\" getting status of volume group "
                         "\"%s\" from kernel\n\n",
                         cmd, lvm_error ( ret), vg_name);
@@ -574,78 +581,78 @@ int main ( int argc, char **argv) {
       }
    }
 
-   if ( opt_L > 0 || opt_l > 0) {
-      if ( opt_L > 0) size *= 2;
-      else            size *= vg_core->pe_size;
-      if ( size % vg_core->pe_size > 0) {
-         size += vg_core->pe_size - size % vg_core->pe_size;
-         printf ( "%s -- rounding up size to physical extent boundary \"%s\"\n",
-                  cmd, ( dummy = lvm_show_size ( size / 2, SHORT)));
-         free ( dummy); dummy = NULL;
-      }
+   if ( opt_L > 0) size *= 2LL;
+   else            size *= vg_core->pe_size;
 
-      size_rest = size % ( stripes * vg_core->pe_size);
-      if ( size_rest != 0) {
-         printf ( "%s -- rounding %d KB to stripe boundary size ",
-                  cmd, size / 2);
-         size = size - size_rest + stripes * vg_core->pe_size;
-         printf ( "%d KB / %u PE\n", size / 2, size / vg_core->pe_size);
-      }
-      if ( opt_s > 0) {
-         if ( opt_v > 0)
-            printf ( "%s -- getting logical volume status from kernel\n", cmd);
-         if ( lvm_tab_lv_check_exist ( lv_name) != TRUE) {
-            fprintf ( stderr,
-                      "%s -- original logical volume \"%s\" doesn't exist\n",
-                      cmd, lv_name);
-            usage ( LVM_ELVCREATE_LV_CHECK_EXIST);
-         }
-         if ( ( ret = lv_status_byname ( vg_name, lv_name, &lv)) < 0) {
-            fprintf ( stderr, "%s -- ERROR \"%s\" getting status of logical "
-                              "volume \"%s\"\n\n",
-                              cmd, lvm_error ( ret), lv_name);
-            return LVM_ELVCREATE_LV_STATUS_BYNAME;
-         }
-         if ( lv->lv_access & LV_SNAPSHOT) {
-            fprintf ( stderr, "%s -- ERROR: can't create snapshot of "
-                              "snapshot \"%s\"\n"
-                              "%s -- please use original logical volume name\n",
-                              cmd, lv_name, cmd);
-            usage ( LVM_EINVALID_CMD_LINE);
-         }
-         if ( size > lv->lv_size * 1.1) {
-            fprintf ( stderr, "%s -- ERROR: size of snapshot is too large\n",
-                              cmd);
-            return LVM_EINVALID_CMD_LINE;
-         } else if ( size < lv->lv_size) {
-            printf ( "%s -- WARNING: the snapshot will be automatically "
-                     "disabled once it gets full\n", cmd);
-         
-         }
-      }
-   } else if ( opt_s == 0) {
-      fprintf ( stderr, "%s -- please give an l or L option\n", cmd);
-      usage ( LVM_EINVALID_CMD_LINE);
+   size_rest = size % vg_core->pe_size;
+   if ( size_rest > 0) {
+      printf ( "%s -- rounding size up to physical extent boundary\n", cmd);
+      size += vg_core->pe_size - size_rest;
    }
 
-   if ( opt_v > 0)
-      printf ( "%s -- reading volume group data of \"%s\"\n", cmd, vg_name);
-   if ( ( ret = lvm_tab_vg_read_with_pv_and_lv ( vg_name, &vg)) != 0) {
-      fprintf ( stderr,
-                "%s -- ERROR \"%s\" couldn't get volume group data of \"%s\"\n",
-                cmd, lvm_error ( ret), vg_name);
-      return LVM_ELVCREATE_VG_READ;
+   /* So that -L2t can be given */
+   if ( size > LVM_LV_SIZE_2TB( vg_core)) size -= vg_core->pe_size;
+
+   if ( stripes > 1) {
+      size_rest = size % ( stripes * vg_core->pe_size);
+      if ( size_rest != 0LL) {
+         printf ( "%s -- rounding to stripe boundary size\n", cmd);
+         size = size - size_rest + stripes * vg_core->pe_size;
+      }
+      if ( size > LVM_LV_SIZE_2TB( vg_core))
+         size -= stripes * vg_core->pe_size;
    }
 
    if ( opt_v > 0)
       printf ( "%s -- checking logical volume maximum size\n", cmd);
-   if ( size > LVM_LV_SIZE_MAX( vg)) {
-      fprintf ( stderr, "%s -- size %d is larger than maximum VGDA "
-                        "kernel size of %llu\n",
-                        cmd, size / 2, LVM_LV_SIZE_MAX( vg) / 2);
+   if ( size > LVM_LV_SIZE_2TB( vg_core)) {
+      fprintf ( stderr, "%s -- size %Ld KB is larger than maximum VGDA "
+                        "kernel size of %Ld KB\n",
+                        cmd, size / 2, LVM_LV_SIZE_2TB( vg_core) / 2);
       usage ( LVM_ELVCREATE_LV_SIZE);
    }
 
+   if ( opt_s > 0) {
+      if ( opt_v > 0)
+         printf ( "%s -- getting logical volume status from kernel\n", cmd);
+      if ( lvm_tab_lv_check_exist ( lv_name) != TRUE) {
+         fprintf ( stderr,
+                   "%s -- original logical volume \"%s\" doesn't exist\n",
+                   cmd, lv_name);
+         usage ( LVM_ELVCREATE_LV_CHECK_EXIST);
+      }
+      if ( ( ret = lv_status_byname ( vg_name, lv_name, &lv)) < 0) {
+         fprintf ( stderr, "%s -- ERROR \"%s\" getting status of logical "
+                           "volume \"%s\"\n\n",
+                           cmd, lvm_error ( ret), lv_name);
+         return LVM_ELVCREATE_LV_STATUS_BYNAME;
+      }
+      if ( lv->lv_access & LV_SNAPSHOT) {
+         fprintf ( stderr, "%s -- ERROR: can't create snapshot of "
+                           "snapshot \"%s\"\n"
+                           "%s -- please use original logical volume name\n",
+                           cmd, lv_name, cmd);
+         usage ( LVM_EINVALID_CMD_LINE);
+      }
+      if ( size > lv->lv_size * 1.1) {
+         fprintf ( stderr, "%s -- ERROR: size of snapshot is too large\n",
+                           cmd);
+         return LVM_EINVALID_CMD_LINE;
+      } else if ( size < lv->lv_size) {
+         printf ( "%s -- WARNING: the snapshot will be automatically "
+                  "disabled once it gets full\n", cmd);
+      }
+   }
+
+   if ( opt_v > 0)
+      printf ( "%s -- reading volume group data of \"%s\"\n", cmd, vg_name);
+   if ( ( ret = lvm_tab_vg_read_with_pv_and_lv ( vg_name, &vg)) < 0) {
+      fprintf ( stderr,
+                "%s -- ERROR \"%s\" couldn't get volume group data of \"%s\"\n",
+                cmd, lvm_error ( ret), vg_name);
+      return LVM_ELVCREATE_VG_READ;
+   }
+
    if ( opt_v > 0) printf ( "%s -- checking volume group free space\n", cmd);
    pe_free = vg->pe_total - vg->pe_allocated;
    if ( pe_free == 0) {
@@ -717,7 +724,8 @@ int main ( int argc, char **argv) {
 
    /* setup logical volume */
    if ( opt_v > 0) printf ( "%s -- setting up logical volume\n", cmd);
-   if ( ( ret = lv_setup_for_create ( vg_name, &vg, lv_name_ptr, &l, size,
+   if ( ( ret = lv_setup_for_create ( vg_name, &vg, lv_name_ptr, &l,
+                                      ( unsigned int) size,
                                       stripes, stripesize, mirrors,
                                       allocation, permission,
                                       pv_allowed)) < 0) {
