@@ -1,12 +1,14 @@
 /*
  * tools/lvextend.c
  *
- * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Sistina Software
+ * Copyright (C) 1997 - 2002  Heinz Mauelshagen, Sistina Software
  *
  * April-May 1997
  * May,June,September 1998
  * February,August,October 1999
  * February,July,November 2000
+ * January,July 2001
+ * February 2002
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,6 +44,11 @@
  *    15/02/2000 - use lvm_error()
  *    01/07/2000 - support for snapshot logical volumes
  *    07/11/2000 - used LVM_VG_NAME environment variable
+ *    23/01/2001 - added call to lvm_init (JT)
+ *    25/07/2001 - avoid misleading "(null)" in error message (AGK)
+ *    27/07/2001 - correct order of args in printf (AGK)
+ *    07/02/2002 - fixes for > 1TB
+ *    14/02/2002 - use lvm_check_number_ll() for (long long) numbers
  *
  */
 
@@ -73,19 +80,13 @@ int main ( int argc, char **argv) {
    int ret = 0;
    int sign = 0;
    int size_rest = 0;
-   long new_size = 0;
+   long long new_size = 0;
    char buffer[NAME_LEN];
    char *dummy = 0;
-#ifdef DEBUG
-   char *options = "A:dh?l:L:v";
-#else
-   char *options = "A:h?l:L:v";
-#endif
+   char *options = "A:h?l:L:v" DEBUG_SHORT_OPTION;
    struct option long_options[] = {
       { "autobackup", required_argument, NULL, 'A'},
-#ifdef DEBUG
-      { "debug",      no_argument,       NULL, 'd'},
-#endif
+      DEBUG_LONG_OPTION
       { "help",       no_argument,       NULL, 'h'},
       { "extents",    required_argument, NULL, 'l'},
       { "size",       required_argument, NULL, 'L'},
@@ -98,9 +99,9 @@ int main ( int argc, char **argv) {
    lv_t *lv = NULL;
    vg_t *vg = NULL;
 
+   /* lvm_init(argc, argv); */
    cmd = basename ( argv[0]);
 
-   SUSER_CHECK;
    LVMTAB_CHECK;
 
    while ( ( c = getopt_long ( argc, argv, options,
@@ -121,15 +122,7 @@ int main ( int argc, char **argv) {
             }
             break;
 
-#ifdef DEBUG
-         case 'd':
-            if ( opt_d > 0) {
-               fprintf ( stderr, "%s -- d option already given\n\n", cmd);
-               return LVM_EINVALID_CMD_LINE;
-            }
-            opt_d++;
-            break;
-#endif
+         DEBUG_HANDLE_CASE_D;
 
          case 'h':
          case '?':
@@ -138,10 +131,8 @@ int main ( int argc, char **argv) {
                      "---------\n\n"
                      "%s\n"
                      "\t[-A/--autobackup y/n]\n"
-#ifdef DEBUG
-                     "\t[-d/--debug]\n"
-#endif
-                     "\t[-h/-?/--help]\n"
+                     DEBUG_HELP
+                     "\t[-h/--help]\n"
                      "\t{-l/--extents [+]LogicalExtentsNumber |\n"
                      "\t -L/--size [+]LogicalVolumeSize[kKmMgGtT]}\n"
                      "\t[-v/--verbose]\n"
@@ -159,7 +150,7 @@ int main ( int argc, char **argv) {
 
             sign = 0;
             if ( *optarg == '+') { sign = 1; optarg++;}
-            if ( ( new_size = lvm_check_number ( optarg, FALSE)) > 0) {
+            if ( ( new_size = lvm_check_number_ll ( optarg, FALSE)) > 0) {
                opt_l++;
             } else {
                fprintf ( stderr, "%s -- ERROR option l argument \"%s\"\n\n",
@@ -177,7 +168,7 @@ int main ( int argc, char **argv) {
 
             sign = 0;
             if ( *optarg == '+') { sign = 1; optarg++;}
-            if ( ( new_size = lvm_check_number ( optarg, TRUE)) > 0) {
+            if ( ( new_size = lvm_check_number_ll ( optarg, TRUE)) > 0) {
                opt_L++;
             } else {
                fprintf ( stderr, "%s -- ERROR option L argument \"%s\"\n\n",
@@ -208,8 +199,22 @@ int main ( int argc, char **argv) {
       fprintf ( stderr, "%s -- please enter a logical volume path\n\n", cmd);
       return LVM_ELVEXTEND_LV_MISSING;
    }
+
+   if ( opt_L + opt_l == 0) {
+      fprintf ( stderr, "%s -- please enter l or L option\n\n", cmd);
+      return LVM_EINVALID_CMD_LINE;
+   }
+
    lv_name = argv[optind];
    LVM_CHECK_DEFAULT_VG_NAME ( lv_name, buffer, sizeof ( buffer));
+
+   if ( lv_name == NULL) {
+      fprintf ( stderr, "%s -- a path needs supplying for "
+                        "logical volume argument \"%s\"\n\n",
+                         cmd, argv[optind]);
+      return LVM_ELVEXTEND_LV_NAME;
+   }
+
    optind++;
 
    /* physical volumes in command line */
@@ -259,7 +264,7 @@ int main ( int argc, char **argv) {
    if ( opt_v > 0) printf ( "%s -- reading volume group data of volume "
                             "group \"%s\" from disk(s)\n",
                             cmd, vg_name);
-   if ( ( ret = lvm_tab_vg_read_with_pv_and_lv ( vg_name, &vg)) != 0) {
+   if ( ( ret = lvm_tab_vg_read_with_pv_and_lv ( vg_name, &vg)) < 0) {
       fprintf ( stderr, "%s -- can't extend logical volume:"
                         " couldn't read data of volume group \"%s\"\n\n",
                         cmd, vg_name);
@@ -270,7 +275,7 @@ int main ( int argc, char **argv) {
       for ( pa = 0; pv_allowed[pa] != NULL; pa++) {
          if ( opt_v > 0) printf ( "%s -- checking physical volume \"%s\" "
                                   "against volume group\n",
-                                  pv_allowed[pa], cmd);
+                                  cmd, pv_allowed[pa]);
          if ( pv_check_in_vg ( vg, pv_allowed[pa]) == FALSE) {
             fprintf ( stderr, "%s -- physical volume \"%s\" doesn't belong "
                               "to volume group \"%s\"\n\n",
@@ -299,39 +304,48 @@ int main ( int argc, char **argv) {
       return LVM_ELVEXTEND_LV_STATUS_BYNAME;
    }
 
-   if ( opt_L + opt_l > 0) {
-      if ( opt_L > 0) new_size *= 2;
-      else            new_size *= vg->pe_size;
-      if ( new_size % vg->pe_size > 0) {
-         new_size += vg->pe_size;
-         printf ( "%s -- rounding size to physical extent boundary\n", cmd);
-      }
-      new_size = new_size / vg->pe_size * vg->pe_size;
-      if ( sign != 0) new_size += vg->lv[l]->lv_size;
+   if ( opt_L > 0) new_size *= 2LL;
+   else            new_size *= vg->pe_size;
 
-      size_rest = new_size % ( vg->lv[l]->lv_stripes * vg->pe_size);
+   size_rest = new_size % vg->pe_size;
+   if ( size_rest > 0) {
+      printf ( "%s -- rounding %s size up to physical extent boundary\n",
+               cmd, sign > 0 ? "relative" : "");
+      new_size += vg->pe_size - size_rest;
+   }
+
+   if ( sign != 0) new_size += vg->lv[l]->lv_size;
+
+   /* So that -L2t can be given */
+   if ( new_size / vg->pe_size < LVM_PE_T_MAX
+        && new_size > LVM_LV_SIZE_2TB( vg)) new_size -= vg->pe_size;
+
+   if ( vg->lv[l]->lv_stripes > 1) {
+      long stripepesize = vg->lv[l]->lv_stripes * vg->pe_size;
+      size_rest = new_size % stripepesize;
       if ( size_rest != 0) {
-         printf ( "%s -- rounding size %ld KB to stripe boundary size ",
-                  cmd, new_size / 2);
-         new_size = new_size - size_rest +
-                    vg->lv[l]->lv_stripes * vg->pe_size;
-         printf ( "%ld KB\n", new_size / 2);
+         printf ( "%s -- rounding size to stripe boundary size\n", cmd);
+         new_size = new_size - size_rest + stripepesize;
       }
+      if ( new_size > LVM_LV_SIZE_2TB(vg)) new_size -= stripepesize;
+   }
 
+   if ( new_size > LVM_LV_SIZE_2TB(vg)) {
+      fprintf ( stderr, "%s -- size %Ld KB is larger than maximum VGDA "
+                        "kernel size of %Ld KB\n",
+                        cmd, new_size / 2, LVM_LV_SIZE_2TB( vg) / 2);
+      return LVM_ELVEXTEND_LV_SIZE;
+   }
 
-      if ( new_size <= vg->lv[l]->lv_size) {
-         fprintf ( stderr, "%s -- new size %s is not bigger than old size\n\n",
-                   cmd, ( dummy = lvm_show_size ( new_size / 2, SHORT)));
-         free ( dummy);
-         return LVM_ELVEXTEND_LV_SIZE;
-      }
-   } else {
-      fprintf ( stderr, "%s -- please give an l or L option\n\n", cmd);
-      return LVM_EINVALID_CMD_LINE;
+   if ( new_size <= vg->lv[l]->lv_size) {
+      fprintf ( stderr, "%s -- new size %s is not bigger than old size\n\n",
+                cmd, ( dummy = lvm_show_size ( new_size / 2, SHORT)));
+      free ( dummy);
+      return LVM_ELVEXTEND_LV_SIZE;
    }
 
    if ( new_size - vg->lv[l]->lv_size >
-        ( vg->pe_total - vg->pe_allocated) * vg->pe_size) {
+        ( ( long long) vg->pe_total - vg->pe_allocated) * vg->pe_size) {
       if ( vg->pe_total - vg->pe_allocated == 0) {
          fprintf ( stderr, "%s -- no free physical extents in volume "
                            "group \"%s\"\n\n",
@@ -346,17 +360,11 @@ int main ( int argc, char **argv) {
       return LVM_ELVEXTEND_LV_SIZE_FREE;
    }
 
-   if ( new_size > LVM_LV_SIZE_MAX( vg)) {
-      fprintf ( stderr, "%s -- new size %ld is to large for VGDA in kernel\n\n",
-                cmd, new_size / 2);
-      return LVM_ELVEXTEND_LV_SIZE_MAX;
-   }
-
    if ( lv->lv_access & LV_SNAPSHOT) {
-      ulong lv_org_size = vg->lv[lv_get_index_by_minor ( vg, vg->lv[l]->lv_snapshot_minor)]->lv_size;
-
+      long long lv_org_size = vg->lv[lv_get_index_by_minor ( vg, vg->lv[l]->lv_snapshot_minor)]->lv_size;
       if ( new_size > lv_org_size * 1.1) {
-            fprintf ( stderr, "%s -- ERROR: size of snapshot is too large\n",
+            fprintf ( stderr, "%s -- ERROR: size of snapshot is larger than "
+                              "the original logical volume\n",
                               cmd);
             return LVM_EINVALID_CMD_LINE;
       }
@@ -369,7 +377,7 @@ int main ( int argc, char **argv) {
    if ( opt_v > 0) printf ( "%s -- setting up logical volume \"%s\" "
                             "for extend\n", cmd, lv_name);
    if ( ( ret = lv_setup_for_extend ( vg_name, vg, lv_name,
-                                      new_size, pv_allowed)) < 0) {
+                                      ( uint) new_size, pv_allowed)) < 0) {
       if ( ret == -LVM_ESIZE) {
          if ( ( vg->lv[l]->lv_allocation & LV_CONTIGUOUS) &&
               pv_allowed == NULL) {
@@ -408,7 +416,7 @@ int main ( int argc, char **argv) {
    /* create it in kernel */
    if ( opt_v > 0) printf ( "%s -- extending logical volume \"%s\" in "
                             "VGDA of kernel\n", cmd, lv_name);
-   if ( ( ret = lv_extend ( vg, vg->lv[l], lv_name)) != 0) {
+   if ( ( ret = lv_extend ( vg, vg->lv[l], lv_name)) < 0) {
       fprintf ( stderr, "%s -- ERROR \"%s\" extending logical volume \"%s\" "
                         " in kernel's VGDA\n\n", 
                         cmd, lvm_error ( ret), lv_name);
@@ -417,7 +425,7 @@ int main ( int argc, char **argv) {
 
    if ( opt_v > 0) printf ( "%s -- storing data of volume group \"%s\" "
                             "on disk(s)\n", cmd, vg_name);
-   if ( ( ret = vg_write_with_pv_and_lv ( vg)) != 0) {
+   if ( ( ret = vg_write_with_pv_and_lv ( vg)) < 0) {
       fprintf ( stderr, "%s -- ERROR \"%s\" storing data of volume group "
                         "\"%s\" on disk(s)\n\n", 
                         cmd, lvm_error ( ret), vg_name);
