@@ -1,12 +1,13 @@
 /*
  * tools/lvmchange.c
  *
- * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Sistina Software
  *
  * March-May 1997
  * June,August 1998
  * February,October 1999
  * February 2000
+ * February 2001
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,6 +37,8 @@
  *    21/02/1999 - removed LVM_LOCK and LVM_UNLOCK
  *    06/10/1999 - implemented support for long options
  *    15/02/2000 - use lvm_error()
+ *    23/01/2001 - added call to lvm_init (JT)
+ *    07/02/2001 - added i option to display the io protocol version
  *
  */
 
@@ -52,41 +55,28 @@ int main ( int argc, char **argv) {
    int c1 = 0;
    int lvm = -1;
    int opt_f = 0;
+   int opt_i = 0;
    int opt_R = 0;
    int opt_v = 0;
    int ret = 0;
-#ifdef DEBUG
-   char *options = "dfh?Rv";
-#else
-   char *options = "fh?Rv";
-#endif
+   char *options = "fh?iRv" DEBUG_SHORT_OPTION;
    struct option long_options[] = {
-#ifdef DEBUG
-      { "debug",      no_argument,       NULL, 'd'},
-#endif
+      DEBUG_LONG_OPTION
       { "force",      no_argument,       NULL, 'f'},
       { "help",       no_argument,       NULL, 'h'},
+      { "iop_version",no_argument,       NULL, 'i'},
       { "reset",      no_argument,       NULL, 'R'},
       { "verbose",    no_argument,       NULL, 'v'},
       { NULL, 0, NULL, 0}
    };
 
+   /* lvm_init(argc, argv); */
    cmd = basename ( argv[0]);
 
-   SUSER_CHECK;
-
    while ( ( c = getopt_long ( argc, argv, options,
                                long_options, NULL)) != EOF) {
       switch ( c) {
-#ifdef DEBUG
-         case 'd':
-            if ( opt_d > 0) {
-               fprintf ( stderr, "%s -- d option already given\n\n", cmd);
-               return LVM_EINVALID_CMD_LINE;
-            }
-            opt_d++;
-            break;
-#endif
+         DEBUG_HANDLE_CASE_D;
 
          case 'f':
             if ( opt_f > 0) {
@@ -96,17 +86,25 @@ int main ( int argc, char **argv) {
             opt_f++;
             break;
 
+         case 'i':
+            if ( opt_i > 0) {
+               fprintf ( stderr, "%s -- i option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
+            }
+            opt_i++;
+            break;
+
+
          case 'h':
          case '?':
             printf ( "\n%s (IOP %d)\n\n%s -- Logical Volume Manager Change\n\n"
                      "Synopsis:\n"
                      "---------\n\n"
                      "%s\n"
-#ifdef DEBUG
-                     "\t[-d/--debug]\n"
-#endif
+                     DEBUG_HELP
                      "\t[-f/--force]\n"
-                     "\t[-h/-?/--help]\n"
+                     "\t[-h/--help]\n"
+                     "\t[-i/--iop_version]\n"
                      "\t[-R/--reset]\n"
                      "\t[-v/--verbose]\n\n",
                      lvm_version, LVM_LIB_IOP_VERSION,  cmd, cmd);
@@ -138,6 +136,16 @@ int main ( int argc, char **argv) {
   
    CMD_MINUS_CHK;
 
+   if (opt_i && opt_R) {
+      fprintf ( stderr, "%s -- option i and R are exclusive\n\n", cmd);
+      return LVM_EINVALID_CMD_LINE;
+   }
+
+   if ( opt_i > 0) {
+      printf ( "%s -- IO protocol version: %d\n\n", cmd, lvm_get_iop_version());
+      return 0;
+   }
+
    if ( opt_R == 0) {
       fprintf ( stderr, "%s -- please give the R option\n\n", cmd);
       return LVM_EINVALID_CMD_LINE;
