@@ -1,11 +1,13 @@
 /*
  * tools/lvremove.c
  *
- * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2001  Heinz Mauelshagen, Sistina Software
  *
  * March 1997
  * April-June,September 1998
- * January 1999
+ * January,July,October 1999
+ * February,November 2000
+ * January,July 2001
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,6 +34,12 @@
  *    27/06/1998 - changed lvm_tab_* calling convention
  *    04/09/1998 - corrected some messages
  *    12/01/1999 - don't continue with next LV in case of an erro with one LV
+ *    39/07/1999 - support snapshot logical volume deletion
+ *    06/10/1999 - implemented support for long options
+ *    15/02/2000 - use lvm_error()
+ *    07/11/2000 - used LVM_VG_NAME environment variable
+ *    23/01/2001 - added call to lvm_init (JT)
+ *    25/07/2001 - avoid misleading "(null)" in error message (AGK)
  *
  */
 
@@ -49,27 +57,35 @@ int main ( int argc, char **argv)
    int c = 0;
    int c1 = 0;
    int opt_A = 1;
+   int opt_A_set = 0;
    int opt_f = 0;
    int opt_v = 0;
    int ret = 0;
-#ifdef DEBUG
-   char *options = "A:dfh?v";
-#else
-   char *options = "A:fh?v";
-#endif
+   char buffer[NAME_LEN];
+   char *options = "A:fh?v" DEBUG_SHORT_OPTION;
+   struct option long_options[] = {
+      { "autobackup", no_argument, NULL, 'A'},
+      DEBUG_LONG_OPTION
+      { "force",      no_argument, NULL, 'f'},
+      { "help",       no_argument, NULL, 'h'},
+      { "verbose",    no_argument, NULL, 'v'},
+      { NULL,         0,           NULL, 0}
+   };
    char *vg_name = NULL;
    char *lv_name = NULL;
    vg_t *vg = NULL;
    lv_t *lv = NULL;
 
+   /* lvm_init(argc, argv); */
    cmd = basename ( argv[0]);
 
-   SUSER_CHECK;
    LVMTAB_CHECK;
 
-   while ( ( c = getopt ( argc, argv, options)) != EOF) {
+   while ( ( c = getopt_long ( argc, argv, options,
+                               long_options, NULL)) != EOF) {
       switch ( c) {
          case 'A':
+            opt_A_set++;
             if ( opt_A > 1) {
                fprintf ( stderr, "%s -- A option already given\n\n", cmd);
                return LVM_EINVALID_CMD_LINE;
@@ -83,31 +99,21 @@ int main ( int argc, char **argv)
             }
             break;
 
-#ifdef DEBUG
-         case 'd':
-            if ( opt_d > 0) {
-               fprintf ( stderr, "%s -- d option already given\n\n", cmd);
-               return LVM_EINVALID_CMD_LINE;
-            }
-            opt_d++;
-            break;
-#endif
+         DEBUG_HANDLE_CASE_D;
 
          case 'h':
          case '?':
-            printf ( "\n%s\n\n%s -- Logical Volume Remove\n\n"
+            printf ( "\n%s (IOP %d)\n\n%s -- Logical Volume Remove\n\n"
                      "Synopsis:\n"
                      "---------\n\n"
                      "%s\n"
-                     "\t[-A y/n]\n"
-#ifdef DEBUG
-                     "\t[-d]\n"
-#endif
-                     "\t[-f]\n"
-                     "\t[-h/?]\n"
-                     "\t[-v]\n"
-                     "\tLogicalVolumePath [LogicalVolumePath...]\n\n",
-                     lvm_version, cmd, cmd);
+                     "\t[-A/--autobackup y/n]\n"
+                     DEBUG_HELP		     
+                     "\t[-f/--force]\n"
+                     "\t[-h/--help]\n"
+                     "\t[-v/--verbose]\n"
+                     "\tLogicalVolume[Path] [LogicalVolume[Path]...]\n\n",
+                     lvm_version, LVM_LIB_IOP_VERSION,  cmd, cmd);
             return 0;
             break;
 
@@ -135,6 +141,7 @@ int main ( int argc, char **argv)
    }
 
    CMD_MINUS_CHK;
+   CMD_CHECK_OPT_A_SET;
 
    if ( optind == argc) {
       fprintf ( stderr, "%s -- please enter a logical volume path\n\n", cmd);
@@ -147,6 +154,15 @@ int main ( int argc, char **argv)
    for ( ; optind < argc; optind++) {
       lv_name = argv[optind];
    
+      LVM_CHECK_DEFAULT_VG_NAME ( lv_name, buffer, sizeof ( buffer));
+
+      if ( lv_name == NULL) {
+         fprintf ( stderr, "%s -- a path needs supplying for "
+                           "logical volume argument \"%s\"\n\n",
+                           cmd, argv[optind]);
+         return LVM_ELVREMOVE_LV_CHECK_NAME;
+      }
+
       if ( opt_v > 0) printf ( "%s -- checking logical volume name \"%s\"\n",
                                cmd, lv_name);
       if ( ( ret = lv_check_name ( lv_name)) < 0) {
@@ -154,9 +170,11 @@ int main ( int argc, char **argv)
                            "name \"%s\"\n\n", cmd, lv_name);
          return LVM_ELVREMOVE_LV_CHECK_NAME;
       }
+
       /* does VG exist? */
       vg_name = vg_name_of_lv ( lv_name);
 
+
       if ( opt_v > 0) printf ( "%s -- checking existence of volume "
                                "group \"%s\"\n",
                                cmd, vg_name);
@@ -190,25 +208,36 @@ int main ( int argc, char **argv)
             fprintf ( stderr, "%s -- logical volume \"%s\" doesn't exist\n\n",
                               cmd, lv_name);
          } else {
-            fprintf ( stderr, "%s -- ERROR %d getting status"
+            fprintf ( stderr, "%s -- ERROR \"%s\" getting status"
                               " of logical volume \"%s\"\n\n",
-                              cmd, ret, lv_name);
+                              cmd, lvm_error ( ret), lv_name);
          }
          return LVM_ELVREMOVE_LV_STATUS;
       }
    
+      if ( lv->lv_access & LV_SNAPSHOT_ORG) {
+         fprintf ( stderr, "%s -- can't remove logical volume \"%s\" "
+                           "under snapshot\n\n",
+                           cmd,
+                           lv_name);
+         return LVM_ELVREMOVE_LV_SNAPSHOT;
+      }
+
       if ( lv->lv_open > 0) {
-         fprintf ( stderr, "%s -- can't remove open logical volume \"%s\"\n\n",
-                           cmd, lv_name);
+         fprintf ( stderr, "%s -- can't remove open %s "
+                           "logical volume \"%s\"\n\n",
+                           cmd,
+                           lv->lv_access & LV_SNAPSHOT ? "snapshot" : "",
+                           lv_name);
          return LVM_ELVREMOVE_LV_OPEN;
       }
    
       if ( opt_v > 0) printf ( "%s -- reading data of volume group \"%s\" "
                                "from disk(s)\n", cmd, lv_name);
       if ( ( ret = lvm_tab_vg_read_with_pv_and_lv ( vg_name, &vg)) != 0) {
-         fprintf ( stderr, "%s -- ERROR %d reading data of volume "
+         fprintf ( stderr, "%s -- ERROR \"%s\" reading data of volume "
                            "group \"%s\"\n\n",
-                           cmd, ret, vg_name);
+                           cmd, lvm_error ( ret), vg_name);
          return LVM_ELVREMOVE_VG_READ;
       }
 
@@ -232,38 +261,39 @@ int main ( int argc, char **argv)
       if ( opt_v > 0) printf ( "%s -- releasing logical volume \"%s\"\n",
                                cmd, lv_name);
       if ( ( ret = lv_release ( vg, lv_name)) != 0) {
-         fprintf ( stderr, "%s -- ERROR %d releasing logical "
+         fprintf ( stderr, "%s -- ERROR \"%s\" releasing logical "
                            "volume \"%s\"\n\n", 
-                           cmd, ret, lv_name);
+                           cmd, lvm_error ( ret), lv_name);
          return LVM_ELVREMOVE_LV_RELEASE;
       }
    
-   
+
       lvm_dont_interrupt ( 0);
    
       /* remove it from kernel */
       if ( opt_v > 0) printf ( "%s -- removing logical volume from VGDA "
                                "in kernel\n", cmd);
       if ( ( ret = lv_remove ( vg, lv, lv_name)) != 0) {
-         fprintf ( stderr, "%s -- ERROR %d removing logical "
+         fprintf ( stderr, "%s -- ERROR \"%s\" removing logical "
                            "volume \"%s\"\n\n", 
-                           cmd, ret, lv_name);
+                           cmd, lvm_error ( ret), lv_name);
          return -LVM_ELVREMOVE_LV_REMOVE;
       }
-   
+
       if ( opt_v > 0) printf ( "%s -- unlinking special file \"%s\"\n",
          cmd, lv_name);
-      if ( unlink ( lv_name) == -1)
-         fprintf ( stderr, "%s -- ERROR %d unlinking special file \"%s\"\n",
-                           cmd, errno, lv_name);
+
+      if(!lvm_check_devfs() && unlink(lv_name) == -1)
+         fprintf(stderr, "%s -- ERROR \"%s\" unlinking special file \"%s\"\n",
+					  cmd, lvm_error ( errno), lv_name);
    
       /* store it on disks */
       if ( opt_v > 0) printf ( "%s -- storing volume group data on disk(s)\n",
                                cmd);
       if ( ( ret = vg_write_with_pv_and_lv ( vg)) != 0) {
-         fprintf ( stderr, "%s -- ERROR %d storing data of volume group \"%s\" "
-                           "on disks\n\n", 
-                           cmd, ret, vg_name);
+         fprintf ( stderr, "%s -- ERROR \"%s\" storing data of volume "
+                           "group \"%s\" on disks\n\n", 
+                           cmd, lvm_error ( ret), vg_name);
          return LVM_ELVREMOVE_VG_WRITE;
       }
 
@@ -274,6 +304,10 @@ int main ( int argc, char **argv)
          printf ( "%s -- doing automatic backup of volume group \"%s\"\n",
                   cmd, vg_name);
          vg_cfgbackup ( vg_name, VG_BACKUP_DIR, opt_v, vg);
+      } else {
+         printf ( "%s -- WARNING: you don't have an automatic "
+                  "backup of \"%s\"\n",
+                  cmd, vg_name);
       }
 
       lvm_interrupt ();
