@@ -58,28 +58,29 @@ char *cmd = NULL;
 int opt_d = 0;
 #endif
 
-void usage(int ret)
-{
+void usage(int ret) {
    FILE *stream = stderr;
 
    if (ret == 0) {
       stream = stdout;
       printf("%s  (IOP %d)\n\n", lvm_version, LVM_LIB_IOP_VERSION);
-      printf("%s - move extents from physical volume to another\n", cmd);
+      printf("%s - move extents from one physical volume to another\n", cmd);
    }
 
    fprintf(stream, "\n%s "
-           "[-A|--autobackup {y|n}] "
-           DEBUG_HELP_2
-           "[-f|--force]\n\t"
-           "[-i|--ignore_read_errors] "
-           "[-h|--help] "
-           "[-t|--test]\n\t"
-           "[-v[v]|--verbose [--verbose]] "
-           "[--version]\n\t"
-           "[{-n|--name} LogicalVolume[:LogicalExtent[-LogicalExtent]...]]\n\t"
-           "SourcePhysicalVolume[:PhysicalExtent[-PhysicalExtent]...]}\n\t"
-           "[DestinationPhysicalVolume[:PhysicalExtent[-PhysicalExtent]...]...]\n\n",
+                   "[-A|--autobackup {y|n}] "
+                   "\t[-f|--force]\n"
+                   "\t[-i|--ignore_read_errors]\n"
+                   "\t[-h|--help]\n"
+                   "\t[-t|--test]\n"
+                   "\t[-v[v]|--verbose [--verbose]]\n"
+                   "\t[--version]\n"
+                   "\t[{-n|--name} LogicalVolume[:LogicalExtent"
+                   "[-LogicalExtent]...]]\n"
+                   "\tSourcePhysicalVolume[:PhysicalExtent"
+                   "[-PhysicalExtent]...]}\n"
+                   "\t[DestinationPhysicalVolume[:PhysicalExtent"
+                   "[-PhysicalExtent]...]...]\n\n",
            cmd);
    exit(ret);
 }
@@ -234,7 +235,8 @@ int main ( int argc, char **argv) {
 
    if ( opt_v > 0) printf ( "%s -- reading data of source physical volume "
                             "from \"%s\"\n", cmd, src_pv_name);
-   if ( ( ret = pv_read ( src_pv_name, &pv, NULL)) < 0 &&
+   ret = pv_read ( src_pv_name, &pv, NULL);
+   if ( ret < 0 &&
         ret != -LVM_EPV_READ_MD_DEVICE) {
       if ( ret == -LVM_EPV_READ_OPEN) {
          fprintf ( stderr, "%s -- source physical volume \"%s\" "
@@ -250,7 +252,8 @@ int main ( int argc, char **argv) {
    memcpy ( &pv_this, pv, sizeof ( pv_this));
    vg_name = pv_this.vg_name;
 
-   if ( ( ret = pv_check_consistency ( &pv_this)) < 0) {
+   ret = pv_check_consistency ( &pv_this);
+   if ( ret < 0) {
       fprintf ( stderr, "%s -- ERROR \"%s\" physical volume \"%s\" "
                         "is inconsistent\n\n",
                         cmd, lvm_error ( ret), src_pv_name);
@@ -279,7 +282,8 @@ int main ( int argc, char **argv) {
    if ( opt_v > 0) printf ( "%s -- checking volume group consistency "
                             "of \"%s\"\n",
                             cmd, vg_name);
-   if ( ( ret = vg_check_consistency_with_pv_and_lv ( vg)) != 0) {
+   ret = vg_check_consistency_with_pv_and_lv ( vg);
+   if ( ret < 0) {
       fprintf ( stderr, "%s -- ERROR \"%s\" volume group \"%s\" is "
                         "inconsistent\n\n",
                         cmd, lvm_error ( ret), vg_name);
@@ -290,7 +294,8 @@ int main ( int argc, char **argv) {
    if ( opt_v > 0) printf ( "%s -- searching for source physical volume \"%s\" "
                             "in volume group \"%s\"\n",
                             cmd, src_pv_name, vg_name);
-   if ( ( src_pv_index = pv_get_index_by_name ( vg, src_pv_name)) < 0) {
+   src_pv_index = pv_get_index_by_name ( vg, src_pv_name);
+   if ( src_pv_index < 0) {
       fprintf ( stderr, "%s -- couldn't find source physical volume \"%s\" "
                         "in volume group \"%s\"\n\n",
                         cmd, src_pv_name, vg_name);
@@ -358,7 +363,8 @@ int main ( int argc, char **argv) {
                               cmd, argv[i]);
             return LVM_EPVMOVE_PV_CHECK_NAME;
          }
-         if ( ( ret = pv_read ( argv[i], &pv, NULL)) < 0 &&
+         ret = pv_read ( argv[i], &pv, NULL);
+         if ( ret < 0 &&
               ret != -LVM_EPV_READ_MD_DEVICE) {
             if ( ret == -LVM_EPV_READ_OPEN) {
                fprintf ( stderr, "%s -- ERROR: destination physical volume "
@@ -371,7 +377,8 @@ int main ( int argc, char **argv) {
             }
             return LVM_EPVMOVE_PV_READ;
          }
-         if ( ( ret = pv_check_consistency ( pv)) < 0) {
+         ret = pv_check_consistency ( pv);
+         if ( ret < 0) {
             fprintf ( stderr, "%s -- ERROR \"%s\" physical volume \"%s\" "
                               "is inconsistent\n\n",
                               cmd, lvm_error ( ret), argv[i]);
@@ -390,6 +397,11 @@ int main ( int argc, char **argv) {
             return LVM_EPVMOVE_DEST_PES;
          }
          np++;
+         if ( np > vg->pv_act) {
+            fprintf ( stderr, "%s -- too many destination physical volumes "
+                              "on command line\n", cmd);
+            return LVM_EINVALID_CMD_LINE;
+         }
       }
    } else {
       for ( p = 0; p < vg->pv_max && vg->pv[p] != NULL; p++) {
@@ -505,11 +517,13 @@ int main ( int argc, char **argv) {
    pe_moved = 0;
    ret = 1;
    while ( ret > 0) {
-      if ( ( ret = pv_move_pes ( vg, buffer, buffer_size, pv_allowed,
-                                 src_pv_index, lv_index,
-                                 les, source_pes, dest_pes,
-                                 opt_v, opt_t)) < 0) {
-         if ( ret == -LVM_EPV_MOVE_PES_NO_SPACE) {
+      ret = pv_move_pes ( vg, buffer, buffer_size, pv_allowed, src_pv_index,
+                          lv_index, les, source_pes, dest_pes, opt_v, opt_t);
+      if ( ret < 0) {
+         if ( ret == -LVM_EPV_MOVE_PES_SNAPSHOT) {
+            fprintf ( stderr, "%s -- Can't move logical extents of an original "
+                              "or snapshot logical volume\n\n", cmd);
+         } else if ( ret == -LVM_EPV_MOVE_PES_NO_SPACE) {
             fprintf ( stderr, "%s -- not enough free/allocatable "
                               "physical extents\n\n", cmd);
          } else if ( ret == -LVM_EPARAM) {
@@ -528,8 +542,13 @@ int main ( int argc, char **argv) {
          return LVM_EPVMOVE_PV_MOVE_PES;
       }
       pe_moved += ret;
-      if ( lv_index > -1 || vg->pv[src_pv_index]->pe_allocated == 0) break;
-      if ( source_pes != NULL || dest_pes != NULL ) ret = 0;
+      if ( lv_index > -1 ||
+           vg->pv[src_pv_index]->pe_allocated == 0)
+         break;
+
+      if ( source_pes != NULL ||
+           dest_pes != NULL )
+         ret = 0;
    }
 
    free ( buffer);
