@@ -1,11 +1,13 @@
 /*
  * tools/vgextend.c
  *
- * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2002  Heinz Mauelshagen, Sistina Software
  *
  * April 1997
  * May,June,September 1998
- * January 1999
+ * January,October 1999
+ * February 2000
+ * February 2002
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +20,7 @@
  * GNU General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * along with LVM; see the file COPYING.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
@@ -34,6 +36,11 @@
  *    27/06/1998 - changed lvm_tab_* calling convention
  *    05/09/1998 - corrected some messages
  *    15/01/1999 - avoided unecessary volume group exixtence check
+ *    06/10/1999 - implemented support for long options
+ *    15/02/2000 - use lvm_error()
+ *    23/01/2001 - added call to lvm_init (JT)
+ *    19/03/2001 - Improved the phrasing of error message
+ *    07/02/2002 - fixes for > 1TB support
  *
  */
 
@@ -57,6 +64,7 @@ int main ( int argc, char **argv) {
    int np = 0;
    int np_sav = 0;
    int opt_A = 1;
+   int opt_A_set = 0;
    int opt_v = 0;
    int p = 0;
    int ret = 0;
@@ -65,20 +73,25 @@ int main ( int argc, char **argv) {
    char *vg_name = NULL;
    vg_t *vg = NULL;
    pv_t **pv = NULL;
-#ifdef DEBUG
-   char *options = "A:dh?v";
-#else
-   char *options = "A:h?v";
-#endif
+   char *options = "A:h?v" DEBUG_SHORT_OPTION;
+   struct option long_options[] = {
+      { "autobackup", required_argument, NULL, 'A'},
+      DEBUG_LONG_OPTION
+      { "help",       no_argument,       NULL, 'h'},
+      { "verbose",    no_argument,       NULL, 'v'},
+      { NULL,         0,                 NULL, 0},
+   };
 
+   /* lvm_init(argc, argv); */
    cmd = basename ( argv[0]);
 
-   SUSER_CHECK;
    LVMTAB_CHECK;
 
-   while ( ( c = getopt ( argc, argv, options)) != EOF) {
+   while ( ( c = getopt_long ( argc, argv, options,
+                               long_options, NULL)) != EOF) {
       switch ( c) {
          case 'A':
+            opt_A_set++;
             if ( opt_A > 1) {
                fprintf ( stderr, "%s -- A option already given\n\n", cmd);
                return LVM_EINVALID_CMD_LINE;
@@ -92,31 +105,21 @@ int main ( int argc, char **argv) {
             }
             break;
 
-#ifdef DEBUG
-         case 'd':
-            if ( opt_d > 0) {
-               fprintf ( stderr, "%s -- d option already given\n\n", cmd);
-               return LVM_EINVALID_CMD_LINE;
-            }
-            opt_d++;
-            break;
-#endif
+         DEBUG_HANDLE_CASE_D;
  
          case 'h':
          case '?':
-            printf ( "\n%s\n\n%s -- Volume Group Extend\n\n"
+            printf ( "\n%s  (IOP %d)\n\n%s -- Volume Group Extend\n\n"
                      "Synopsis:\n"
                      "---------\n\n"
                      "%s\n"
-                     "\t[-A y/n]\n"
-#ifdef DEBUG
-                     "\t[-d]\n"
-#endif
-                     "\t[-h/-?]\n"
-                     "\t[-v]\n"
+                     "\t[-A/--autobackup y/n]\n"
+                     DEBUG_HELP
+                     "\t[-h/--help]\n"
+                     "\t[-v/--verbose]\n"
                      "\tVolumeGroupName\n"
                      "\tPhysicalDevicePath [PhysicalDevicePath...]\n\n",
-                     lvm_version, cmd, cmd);
+                     lvm_version, LVM_LIB_IOP_VERSION,  cmd, cmd);
             return 0;
             break;
 
@@ -138,7 +141,7 @@ int main ( int argc, char **argv) {
    CMD_MINUS_CHK;
    LVM_CHECK_IOP;
    LVM_LOCK ( 0);
-
+   CMD_CHECK_OPT_A_SET;
 
    if ( optind == argc) {
       fprintf ( stderr, "%s -- please enter a volume group name"
@@ -191,9 +194,9 @@ int main ( int argc, char **argv) {
                             "from lvmtab\n",
                             cmd, vg_name);
    if ( ( ret = lvm_tab_vg_read_with_pv_and_lv ( vg_name, &vg)) < 0) {
-      fprintf ( stderr, "%s -- ERROR %d: can't extend;"
+      fprintf ( stderr, "%s -- ERROR \"%s\" can't extend;"
                         " couldn't get data of volume group \"%s\"\n\n",
-                        cmd, ret, vg_name);
+                        cmd, lvm_error ( ret), vg_name);
       return LVM_EVGEXTEND_VG_READ;
    }
 
@@ -205,7 +208,7 @@ int main ( int argc, char **argv) {
    }
 
    printf ( "%s -- INFO: maximum logical volume size is %s\n",
-            cmd, ( dummy = lvm_show_size ( LVM_LV_SIZE_MAX ( vg) / 2, LONG)));
+            cmd, ( dummy = lvm_show_size ( LVM_LV_SIZE_2TB ( vg) / 2, LONG)));
    free ( dummy); dummy = NULL;
 
 
@@ -219,9 +222,9 @@ int main ( int argc, char **argv) {
    if ( opt_v > 0) printf ( "%s -- reading data for all physical volumes "
                             "from disk(s)\n", cmd);
    if ( ( ret = pv_read_all_pv ( &pv, FALSE)) < 0) {
-      fprintf ( stderr, "%s -- ERROR %d: can't extend;"
+      fprintf ( stderr, "%s -- ERROR \"%s\" can't extend;"
                         " couldn't read physical volume data\n\n",
-                cmd, ret);
+                        cmd, lvm_error ( ret));
       return LVM_EVGEXTEND_PV_READ_ALL_PV;
    }
 
@@ -245,12 +248,12 @@ int main ( int argc, char **argv) {
             fprintf ( stderr, "%s -- ERROR: getting size of "
                               "physical volume \"%s\"\n\n",
                               cmd, error_pv_name);
-      } else if ( ret == -LVM_EVG_SETUP_FOR_EXTEND_PV_ALLREADY) {
+      } else if ( ret == -LVM_EVG_SETUP_FOR_EXTEND_PV_ALREADY) {
             fprintf ( stderr, "%s -- ERROR: physical volume \"%s\" "
                               "already belongs to volume group \"%s\"\n\n",
                               cmd, error_pv_name, vg_name);
       } else if ( ret == -LVM_EVG_SETUP_FOR_EXTEND_PV_CHECK_NEW) {
-            fprintf ( stderr, "%s -- ERROR: \"%s\" is no new physical "
+            fprintf ( stderr, "%s -- ERROR: \"%s\" is not a new physical "
                               "volume\n\n",
                               cmd, error_pv_name);
       } else if ( ret == -LVM_EVG_SETUP_FOR_EXTEND_PV_SIZE_REL) {
@@ -263,8 +266,9 @@ int main ( int argc, char **argv) {
                               "extend volume group \"%s\"\n\n",
                                cmd, vg_name);
       } else {
-         fprintf ( stderr, "%s -- ERROR %d setting up volume group "
-                           "%s for extend\n\n", cmd, ret, vg_name);
+         fprintf ( stderr, "%s -- ERROR \"%s\" setting up volume group "
+                           "%s for extend\n\n",
+                           cmd, lvm_error ( ret), vg_name);
       }
       return LVM_EVGEXTEND_VG_SETUP;
    }
@@ -283,9 +287,9 @@ int main ( int argc, char **argv) {
                                "physical volume \"%s\" in kernel\n",
                                cmd, vg_name, vg->pv[np]->pv_name);
       if ( ( ret = vg_extend ( vg_name, vg->pv[np], vg)) < 0) {
-         fprintf ( stderr, "%s -- ERROR %d extending volume group \"%s\" by "
-                           "physical volume \"%s\" in kernel\n",
-                           cmd, ret, vg_name, vg->pv[np]->pv_name);
+         fprintf ( stderr, "%s -- ERROR \"%s\" extending volume group "
+                           "\"%s\" by physical volume \"%s\" in kernel\n",
+                           cmd, lvm_error ( ret), vg_name, vg->pv[np]->pv_name);
          for ( p = np_sav; p < np; p++) vg_reduce ( vg_name, vg->pv[p], vg);
          return LVM_EVGEXTEND_VG_EXTEND;
       }
@@ -295,8 +299,9 @@ int main ( int argc, char **argv) {
    if ( opt_v > 0) printf ( "%s -- storing volume group data of "
                             "%s on disk(s)\n", cmd, vg_name);
    if ( ( ret = vg_write_with_pv_and_lv ( vg)) < 0) {
-      fprintf ( stderr, "%s -- ERROR %d storing data of volume group "
-                        "\"%s\" on disk(s)\n\n", cmd, ret, vg_name);
+      fprintf ( stderr, "%s -- ERROR \"%s\" storing data of volume group "
+                        "\"%s\" on disk(s)\n\n",
+                        cmd, lvm_error ( ret), vg_name);
       return LVM_EVGEXTEND_VG_WRITE;
    }
 
@@ -306,6 +311,9 @@ int main ( int argc, char **argv) {
       printf ( "%s -- doing automatic backup of volume group \"%s\"\n",
                cmd, vg_name);
       vg_cfgbackup ( vg_name, VG_BACKUP_DIR, opt_v, vg);
+   } else {
+      printf ( "%s -- WARNING: you don't have an automatic backup of \"%s\"\n",
+               cmd, vg_name);
    }
    printf ( "%s -- volume group \"%s\" successfully extended\n", cmd, vg_name);
 
