@@ -1,9 +1,11 @@
 /*
  * tools/vgmerge.c
  *
- * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Sistina Software
  *
  * June,September,December 1998
+ * October 1999
+ * February 2000
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,6 +30,10 @@
  *    05/09/1998 - corrected some messages
  *    22/12/1998 - fixed PE counter bug
  *                 (pv_check_consistency_all_pv() involved too)
+ *    06/10/1999 - implemented support for long options
+ *    15/02/2000 - use lvm_error()
+ *    23/01/2001 - added call to lvm_init (JT)
+ *
  */
 
 #include <lvm_user.h>
@@ -44,6 +50,7 @@ int main ( int argc, char **argv) {
    int np = 0;
    int np_sav = 0;
    int opt_A = 1;
+   int opt_A_set = 0;
    int opt_l = 0;
    int opt_t = 0;
    int opt_v = 0;
@@ -51,11 +58,16 @@ int main ( int argc, char **argv) {
    int pe_allocated_sav = 0;
    int ret = 0;
    int v = 0;
-#ifdef DEBUG
-   char *options = "A:dh?ltv";
-#else
-   char *options = "A:h?ltv";
-#endif
+   char *options = "A:h?ltv" DEBUG_SHORT_OPTION;
+   struct option long_options[] = {
+      { "autobackup", required_argument, NULL, 'A'},
+      DEBUG_LONG_OPTION
+      { "help",       no_argument,       NULL, 'h'},
+      { "list",       no_argument,       NULL, 'l'},
+      { "test",       no_argument,       NULL, 't'},
+      { "verbose",    no_argument,       NULL, 'v'},
+      { NULL,         0,                 NULL, 0},
+   };
    char *vg_name_to = NULL;
    char *vg_name_from = NULL;
    vg_t *vg_to = NULL;
@@ -63,14 +75,16 @@ int main ( int argc, char **argv) {
    vg_t vg_to_this;
    vg_t vg_from_this;
 
+   /* lvm_init(argc, argv); */
    cmd = basename ( argv[0]);
 
-   SUSER_CHECK;
    LVMTAB_CHECK;
 
-   while ( ( c = getopt ( argc, argv, options)) != EOF) {
+   while ( ( c = getopt_long ( argc, argv, options,
+                               long_options, NULL)) != EOF) {
       switch ( c) {
          case 'A':
+	    opt_A_set++;
             if ( opt_A > 1) {
                fprintf ( stderr, "%s -- A option already given\n\n", cmd);
                return LVM_EINVALID_CMD_LINE;
@@ -84,32 +98,22 @@ int main ( int argc, char **argv) {
             }
             break;
 
-#ifdef DEBUG
-         case 'd':
-            if ( opt_d > 0) {
-               fprintf ( stderr, "%s -- d option already given\n\n", cmd);
-               return LVM_EINVALID_CMD_LINE;
-            }
-            opt_d++;
-            break;
-#endif
+         DEBUG_HANDLE_CASE_D;
 
          case 'h':
          case '?':
-            printf ( "\n%s\n\n%s -- Volume Group Merge\n\n"
+            printf ( "\n%s  (IOP %d)\n\n%s -- Volume Group Merge\n\n"
                      "Synopsis:\n"
                      "---------\n\n"
                      "%s\n"
-                     "\t[-A y/n]\n"
-#ifdef DEBUG
-                     "\t[-d]\n"
-#endif
-                     "\t[-h/-?]\n"
-                     "\t[-l]\n"
-                     "\t[-t]\n"
-                     "\t[-v]\n"
+                     "\t[-A/--autobackup y/n]\n"
+                     DEBUG_HELP
+                     "\t[-h/--help]\n"
+                     "\t[-l/--list]\n"
+                     "\t[-t/--test]\n"
+                     "\t[-v/--verbose]\n"
                      "\tDestinationVolumeGroupName SourceVolumeGroupName\n\n",
-                     lvm_version, cmd, cmd);
+                     lvm_version, LVM_LIB_IOP_VERSION,  cmd, cmd);
             return 0;
             break;
 
@@ -147,6 +151,8 @@ int main ( int argc, char **argv) {
    CMD_MINUS_CHK;
    LVM_CHECK_IOP;
    LVM_LOCK ( 0);
+   CMD_CHECK_OPT_A_SET;
+
 
    if ( optind != argc - 2) {
       fprintf ( stderr, "%s -- please enter 2 volume group names\n\n", cmd);
@@ -179,9 +185,9 @@ int main ( int argc, char **argv) {
                             cmd, vg_name_to, LVMTAB);
    if ( ( ret = lvm_tab_vg_read_with_pv_and_lv ( vg_name_to, &vg_to)) != 0) {
       if ( ret != -LVM_EVG_CFGRESTORE_OPEN) {
-         fprintf ( stderr, "%s -- ERROR %d: couldn't get volume "
+         fprintf ( stderr, "%s -- ERROR \"%s\" couldn't get volume "
                            "group data of \"%s\"\n\n",
-                           cmd, ret, vg_name_to);
+                           cmd, lvm_error ( ret), vg_name_to);
       } else fprintf ( stderr, "\n");
       return LVM_EVGMERGE_VG_READ_TO;
    }
@@ -193,9 +199,9 @@ int main ( int argc, char **argv) {
                             cmd, vg_name_from);
    if ( ( ret = lvm_tab_vg_read_with_pv_and_lv ( vg_name_from,
                                                  &vg_from)) < 0) {
-      fprintf ( stderr, "%s -- ERROR %d: couldn't get data of volume "
+      fprintf ( stderr, "%s -- ERROR \"%s\" couldn't get data of volume "
                         "group \"%s\"\n\n",
-                        cmd, ret, vg_name_from);
+                        cmd, lvm_error ( ret), vg_name_from);
       return LVM_EVGMERGE_VG_READ_FROM;
    }
 
@@ -224,9 +230,9 @@ int main ( int argc, char **argv) {
                            "exceeded for volume group \"%s\"\n\n",
                            cmd, vg_name_to);
       } else {
-         fprintf ( stderr, "%s -- ERROR %d merging volume group \"%s\" "
+         fprintf ( stderr, "%s -- ERROR \"%s\" merging volume group \"%s\" "
                            "into \"%s\"\n\n",
-                           cmd, ret, vg_name_from, vg_name_to);
+                           cmd, lvm_error ( ret), vg_name_from, vg_name_to);
       }
       return LVM_EVGMERGE_VG_SETUP;
    }
@@ -246,9 +252,10 @@ int main ( int argc, char **argv) {
          vg_to->pv[np]->pe_allocated = 0;
          if ( opt_t == 0 &&
               ( ret = vg_extend ( vg_name_to, vg_to->pv[np], vg_to)) < 0) {
-            fprintf ( stderr, "%s -- ERROR %d extending volume group \"%s\" "
-                              "by physical volume \"%s\" in kernel\n",
-                              cmd, ret, vg_name_to, vg_to->pv[np]->pv_name);
+            fprintf ( stderr, "%s -- ERROR \"%s\" extending volume group "
+                              "\"%s\" by physical volume \"%s\" in kernel\n",
+                              cmd, lvm_error ( ret),
+                              vg_name_to, vg_to->pv[np]->pv_name);
             for ( p = np_sav; p < np; p++) {
                if ( opt_v > 0) printf ( "%s -- trying to reduce volume group "
                                         "\"%s\" by physical volume \"%s\"\n",
@@ -273,9 +280,10 @@ int main ( int argc, char **argv) {
                                      vg_to->lv[l],
                                      vg_to->lv[l]->lv_name)) < 0) {
                if ( ret != -EEXIST) {
-                  fprintf ( stderr, "%s -- ERROR %d creating logical volume "
-                                    "%s in kernel\n",
-                                    cmd, ret, vg_to->lv[l]->lv_name);
+                  fprintf ( stderr, "%s -- ERROR \"%s\" creating logical "
+                                    "volume %s in kernel\n",
+                                    cmd, lvm_error ( ret),
+                                    vg_to->lv[l]->lv_name);
                }
             }
          }
@@ -287,9 +295,9 @@ int main ( int argc, char **argv) {
                             "%s on disk(s)\n", cmd, vg_name_to);
    if ( opt_t == 0 &&
         ( ret = vg_write_with_pv_and_lv ( vg_to)) < 0) {
-      fprintf ( stderr, "%s -- ERROR %d storing data of volume group "
+      fprintf ( stderr, "%s -- ERROR \"%s\" storing data of volume group "
                         "\"%s\" on disk(s)\n\n",
-                        cmd, ret, vg_name_to);
+                        cmd, lvm_error ( ret), vg_name_to);
       return LVM_EVGMERGE_VG_WRITE;
    }
 
@@ -300,6 +308,9 @@ int main ( int argc, char **argv) {
       printf ( "%s -- doing automatic backup of volume group \"%s\"\n",
                cmd, vg_name_to);
       vg_cfgbackup ( vg_name_to, VG_BACKUP_DIR, opt_v, vg_to);
+   } else {
+      printf ( "%s -- WARNING: you don't have an automatic backup of \"%s\"\n",
+               cmd, vg_name_to);
    }
 
    if ( opt_v > 0) printf ( "%s -- removing special files of volume "
@@ -327,9 +338,9 @@ int main ( int argc, char **argv) {
                             cmd, vg_name_to);
    if ( opt_t == 0 &&
         ( ret = vg_create_dir_and_group_and_nodes ( vg_to, opt_v)) < 0) {
-      fprintf ( stderr, "%s -- ERROR %d: creating volume group "
+      fprintf ( stderr, "%s -- ERROR \"%s\" creating volume group "
                         "nodes of \"%s\"\n\n",
-                        cmd, ret, vg_name_to);
+                        cmd, lvm_error ( ret), vg_name_to);
       return LVM_EVGMERGE_VG_CREATE_DIR_AND_GROUP_NODES;
    }
 
@@ -338,9 +349,9 @@ int main ( int argc, char **argv) {
                             cmd, LVMTAB, vg_name_from);
    if ( opt_t == 0 &&
         ( ret = lvm_tab_vg_remove ( vg_name_from)) < 0) {
-      fprintf ( stderr, "%s -- ERROR %d removing volume group \"%s\" entry "
+      fprintf ( stderr, "%s -- ERROR \"%s\" removing volume group \"%s\" entry "
                         "from \"%s\"\n",
-                        cmd, ret, vg_name_from, LVMTAB);
+                        cmd, lvm_error ( ret), vg_name_from, LVMTAB);
       return LVM_EVGMERGE_VG_REMOVE;
    }
 
