@@ -39,6 +39,7 @@
  *    06/10/1999 - implemented support for long options
  *    15/02/2000 - use lvm_error()
  *    23/01/2001 - added call to lvm_init (JT)
+ *    17/08/2001 - remove unused pv variable
  *
  */
 
@@ -65,7 +66,6 @@ int main ( int argc, char **argv)
    };
    char *vg_name = NULL;
    vg_t *vg = NULL;
-   pv_t **pv = NULL;
 
    /* lvm_init(argc, argv); */
    cmd = basename ( argv[0]);
@@ -84,7 +84,7 @@ int main ( int argc, char **argv)
                      "---------\n\n"
                      "%s\n"
                      DEBUG_HELP
-                     "\t[-h/-?/--help]\n"
+                     "\t[-h/--help]\n"
                      "\t[-v/--verbose]\n"
                      "\tVolumeGroupName [VolumeGroupName...]\n\n",
                      lvm_version, LVM_LIB_IOP_VERSION,  cmd, cmd);
